/*
 * startup_clocks_d51.c
 *
 * Created: 27.08.2018 13:10:42
 *  Author: Thomas F
 */ 



#include <stdbool.h>
#include "samd51.h"
#include "clocks.h"


void SystemInitClock(void)
{
// ----------------------------------------------------------------------------------------------
	
	// Enable module clocks (are on by default)
	//MCLK->APBAMASK.bit.MCLK_    = true;  ;-)
	//MCLK->APBAMASK.bit.GCLK_    = true;
	//MCLK->APBAMASK.bit.OSCCTRL_ = true;
	//MCLK->APBBMASK.bit.NVMCTRL_ = true;

// ----------------------------------------------------------------------------------------------

#ifdef CRYSTALL32K
	// Enable XOSC32K
	OSC32KCTRL->XOSC32K.reg = OSC32KCTRL_XOSC32K_EN32K
							| OSC32KCTRL_XOSC32K_CGM_XT
							| OSC32KCTRL_XOSC32K_XTALEN
							| OSC32KCTRL_XOSC32K_ENABLE;
	
	while(OSC32KCTRL->STATUS.bit.XOSC32KRDY == false) { ; }

	// Enable GEN_GCLK_CR32K
	GCLK->GENCTRL[GEN_GCLK_CR32K].reg = GCLK_GENCTRL_SRC_XOSC32K
									  | GCLK_GENCTRL_GENEN;
#endif

// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_LP32K
	GCLK->GENCTRL[GEN_GCLK_LP32K].reg = GCLK_GENCTRL_SRC_OSCULP32K
									  | GCLK_GENCTRL_GENEN;

 
// ----------------------------------------------------------------------------------------------

#ifndef CRYSTALL32K
	// Enable GEN_GCLK_1M (DFLL48M is the source)
	GCLK->GENCTRL[GEN_GCLK_1M].reg = GCLK_GENCTRL_SRC_DFLL
								   | GCLK_GENCTRL_DIV(48)
								   | GCLK_GENCTRL_GENEN;
#endif
 
// ----------------------------------------------------------------------------------------------
	
	// Enable Peripheral Clocks for FDPLL(0:1)
#ifdef CRYSTALL32K
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL0].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_CR32K)
											  | GCLK_PCHCTRL_CHEN;
	
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL1].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_CR32K)
											  | GCLK_PCHCTRL_CHEN;
#else
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL0].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_1M)
											  | GCLK_PCHCTRL_CHEN;
   
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL1].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_1M)
											  | GCLK_PCHCTRL_CHEN;
#endif

      
	// The clock is only needed if FDPLL(0:1) on-the-fly rotation update is used
	// "..._FDPLL032K" "..._FDPLL132K" are the same values
#ifdef CRYSTALL32K
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL032K].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_CR32K)
												 | GCLK_PCHCTRL_CHEN;
#else
	GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL032K].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_LP32K)	
											     | GCLK_PCHCTRL_CHEN;
#endif
 
	// Wait for clocks are enabled
	while(GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL0].bit.CHEN	== false) { ; }
	while(GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL1].bit.CHEN	== false) { ; }
	while(GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL032K].bit.CHEN == false) { ; }
	
// ----------------------------------------------------------------------------------------------
     
	// Enable DPLL0 120 MHz
#ifdef CRYSTALL32K
	#define LDR0FREF	  (32768u)		//  32 kHz
#else
	#define LDR0FREF	  (1000000u)	//   1 MHz
#endif
	#define LDR0FDES	(120000000u)	// 120 MHz
	
	#define LDR0VAL		((LDR0FDES / LDR0FREF) - 1)
	#define LDR0FRACVAL	((LDR0FDES % LDR0FREF) * 32u / LDR0FREF)
	
	OSCCTRL->Dpll[0].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(LDR0FRACVAL)
								   | OSCCTRL_DPLLRATIO_LDR(LDR0VAL);
   
	while(OSCCTRL->Dpll[0].DPLLSYNCBUSY.bit.DPLLRATIO) { ; }
 	
	OSCCTRL->Dpll[0].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK
								   | OSCCTRL_DPLLCTRLB_LBYPASS;		// MUST USE LBYPASS DUE TO BUG IN REV A OF SAMD51 (if ref clock <= 400kHz)
   
	OSCCTRL->Dpll[0].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
   
	while((OSCCTRL->Dpll[0].DPLLSTATUS.bit.CLKRDY == false) || (OSCCTRL->Dpll[0].DPLLSTATUS.bit.LOCK == false)) { ; }
 
 
	// Set Flash Wait State (is set by default to Auto Wait State)
	//NVMCTRL->CTRLA.bit.AUTOWS = true;


	// Set HS / CPU division factor (are set by default to 1)
	//MCLK->HSDIV.reg  = 1; read only (is fixed 1)
	//MCLK->CPUDIV.reg = 1;
	while(MCLK->INTFLAG.bit.CKRDY == false) { ; }
	
	// Switch Core Clock to 120 MHz
	GCLK->GENCTRL[GEN_GCLK_120M].reg = GCLK_GENCTRL_SRC_DPLL0
									 | GCLK_GENCTRL_GENEN;
	
	SystemCoreClock = LDR0FDES;

// ----------------------------------------------------------------------------------------------
 
	// Enable DPLL1 192 MHz
#ifdef CRYSTALL32K
	#define LDR1FREF	  (32768u)		//  32 kHz
#else
	#define LDR1FREF	  (1000000u)	//   1 MHz
#endif
	#define LDR1FDES	(192000000u)	// 192 MHz

	#define LDR1VAL		((LDR1FDES / LDR1FREF) - 1)
	#define LDR1FRACVAL	((LDR1FDES % LDR1FREF) * 32u / LDR1FREF)
	 
	OSCCTRL->Dpll[1].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(LDR1FRACVAL)
								   | OSCCTRL_DPLLRATIO_LDR(LDR1VAL);
 
	while(OSCCTRL->Dpll[1].DPLLSYNCBUSY.bit.DPLLRATIO) { ; }
 
	OSCCTRL->Dpll[1].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK
								   | OSCCTRL_DPLLCTRLB_LBYPASS;		// MUST USE LBYPASS DUE TO BUG IN REV A OF SAMD51 (if ref clock <= 400kHz)
 
	OSCCTRL->Dpll[1].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
 
	while((OSCCTRL->Dpll[1].DPLLSTATUS.bit.CLKRDY == false) || (OSCCTRL->Dpll[1].DPLLSTATUS.bit.LOCK == false)) { ; }
 

	// Enable GEN_GCLK_192M
	GCLK->GENCTRL[GEN_GCLK_192M].reg = GCLK_GENCTRL_SRC_DPLL1
									 | GCLK_GENCTRL_GENEN;
  	
// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_96M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_96M].reg = GCLK_GENCTRL_SRC_DPLL1
									| GCLK_GENCTRL_DIV(2)
									| GCLK_GENCTRL_GENEN;
 
// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_64M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_64M].reg = GCLK_GENCTRL_SRC_DPLL1
									| GCLK_GENCTRL_DIV(3)
									| GCLK_GENCTRL_GENEN;
 
// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_48M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_48M].reg = GCLK_GENCTRL_SRC_DPLL1
									| GCLK_GENCTRL_DIV(4)
									| GCLK_GENCTRL_GENEN;

// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_32M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_32M].reg = GCLK_GENCTRL_SRC_DPLL1
									| GCLK_GENCTRL_DIV(6)
									| GCLK_GENCTRL_GENEN;

// ----------------------------------------------------------------------------------------------

	// Enable GEN_GCLK_16M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_16M].reg = GCLK_GENCTRL_SRC_DPLL1
									| GCLK_GENCTRL_DIV(12)
									| GCLK_GENCTRL_GENEN;

// ----------------------------------------------------------------------------------------------

#ifdef CRYSTALL32K
	// Enable GEN_GCLK_1M (DPLL1 192MHz is the source)
	GCLK->GENCTRL[GEN_GCLK_1M].reg = GCLK_GENCTRL_SRC_DPLL1
								   | GCLK_GENCTRL_DIV(192)
								   | GCLK_GENCTRL_GENEN;
#endif

// ----------------------------------------------------------------------------------------------
	
}
