/*
 * ARP.c
 *
 * Created: 22.12.2018 16:41:32
 *  Author: Thomas
 */ 


#include <stdbool.h>
#include <string.h>

#include "Internet/Internet.h"
#include "ARP.h"



int ArpReadFrame(uint8_t* buffer[], uint16_t* size, ARP_FRAME_Type* arpFrame)
{
	if (*size < ARP_FRAME_SIZE)
		return -1;
	
	uint8_t* buf = *buffer;
	
	arpFrame->HardwareType	= (buf[0] << 8) | buf[1];
	arpFrame->Protocol		= (buf[2] << 8) | buf[3];
	arpFrame->MacAddrLen	=  buf[4];
	arpFrame->ProtocolLen	=  buf[5];
	
	if (arpFrame->HardwareType != ARP_HW_ETHERNET)
		return -2;
	if (arpFrame->Protocol != ARP_IP)
		return -3;
	if (arpFrame->MacAddrLen != 6)
		return -4;
	if (arpFrame->ProtocolLen != 4)
		return -5;
	
	arpFrame->Operation = (buf[6] << 8) | buf[7];

	arpFrame->SenderMacAddr.BYTE[0] = buf[8];
	arpFrame->SenderMacAddr.BYTE[1] = buf[9];
	arpFrame->SenderMacAddr.BYTE[2] = buf[10];
	arpFrame->SenderMacAddr.BYTE[3] = buf[11];
	arpFrame->SenderMacAddr.BYTE[4] = buf[12];
	arpFrame->SenderMacAddr.BYTE[5] = buf[13];
	
	arpFrame->SenderIpAddr.BYTE[0] = buf[14];
	arpFrame->SenderIpAddr.BYTE[1] = buf[15];
	arpFrame->SenderIpAddr.BYTE[2] = buf[16];
	arpFrame->SenderIpAddr.BYTE[3] = buf[17];
	
	arpFrame->TargetMacAddr.BYTE[0] = buf[18];
	arpFrame->TargetMacAddr.BYTE[1] = buf[19];
	arpFrame->TargetMacAddr.BYTE[2] = buf[20];
	arpFrame->TargetMacAddr.BYTE[3] = buf[21];
	arpFrame->TargetMacAddr.BYTE[4] = buf[22];
	arpFrame->TargetMacAddr.BYTE[5] = buf[23];
	
	arpFrame->TargetIpAddr.BYTE[0] = buf[24];
	arpFrame->TargetIpAddr.BYTE[1] = buf[25];
	arpFrame->TargetIpAddr.BYTE[2] = buf[26];
	arpFrame->TargetIpAddr.BYTE[3] = buf[27];
	
	*buffer	+= ARP_FRAME_SIZE;
	*size	-= ARP_FRAME_SIZE;
	
	return 0;	
}



int ArpWriteFrame(uint8_t* buffer[], uint16_t* size, ARP_FRAME_Type* arpFrame)
{
	if (*size < ARP_FRAME_SIZE)
		return -1;

	if (arpFrame->HardwareType != ARP_HW_ETHERNET)
		return -2;
	if (arpFrame->Protocol != ARP_IP)
		return -3;
	if (arpFrame->MacAddrLen != 6)
		return -4;
	if (arpFrame->ProtocolLen != 4)
		return -5;

	uint8_t* buf = *buffer;
	
	buf[0] = (uint8_t)(arpFrame->HardwareType >> 8);
	buf[1] = (uint8_t)(arpFrame->HardwareType >> 0);

	buf[2] = (uint8_t)(arpFrame->Protocol >> 8);
	buf[3] = (uint8_t)(arpFrame->Protocol >> 0);

	buf[4] = arpFrame->MacAddrLen;

	buf[5] = arpFrame->ProtocolLen;
	
	buf[6] = (uint8_t)(arpFrame->Operation >> 8);
	buf[7] = (uint8_t)(arpFrame->Operation >> 0);

	buf[8] = arpFrame->SenderMacAddr.BYTE[0];
	buf[9] = arpFrame->SenderMacAddr.BYTE[1];
	buf[10] = arpFrame->SenderMacAddr.BYTE[2];
	buf[11] = arpFrame->SenderMacAddr.BYTE[3];
	buf[12] = arpFrame->SenderMacAddr.BYTE[4];
	buf[13] = arpFrame->SenderMacAddr.BYTE[5];
	
	buf[14] = arpFrame->SenderIpAddr.BYTE[0];
	buf[15] = arpFrame->SenderIpAddr.BYTE[1];
	buf[16] = arpFrame->SenderIpAddr.BYTE[2];
	buf[17] = arpFrame->SenderIpAddr.BYTE[3];
	
	buf[18] = arpFrame->TargetMacAddr.BYTE[0];
	buf[19] = arpFrame->TargetMacAddr.BYTE[1];
	buf[20] = arpFrame->TargetMacAddr.BYTE[2];
	buf[21] = arpFrame->TargetMacAddr.BYTE[3];
	buf[22] = arpFrame->TargetMacAddr.BYTE[4];
	buf[23] = arpFrame->TargetMacAddr.BYTE[5];
	
	buf[24] = arpFrame->TargetIpAddr.BYTE[0];
	buf[25] = arpFrame->TargetIpAddr.BYTE[1];
	buf[26] = arpFrame->TargetIpAddr.BYTE[2];
	buf[27] = arpFrame->TargetIpAddr.BYTE[3];
	
	*buffer	+= ARP_FRAME_SIZE;
	*size	-= ARP_FRAME_SIZE;
	
	return 0;
}



int ArpSetFrame( uint16_t operation, INET_MAC_Type* senderMacAddr, INET_IP_Type* senderIpAddr,
					INET_MAC_Type* targetMacAddr, INET_IP_Type* targetIpAddr, ARP_FRAME_Type* arpFrame )
{
	arpFrame->HardwareType	= ARP_HW_ETHERNET;
	arpFrame->Protocol		= ARP_IP;
	arpFrame->MacAddrLen	= 6;
	arpFrame->ProtocolLen	= 4;
	arpFrame->Operation		= operation;
	arpFrame->SenderMacAddr	= *senderMacAddr;
	arpFrame->SenderIpAddr	= *senderIpAddr;
	arpFrame->TargetMacAddr	= *targetMacAddr;
	arpFrame->TargetIpAddr	= *targetIpAddr;
	
	return 0;
}