/*
 * EthernetII.c
 *
 * Created: 23.12.2018 10:56:52
 *  Author: Thomas
 */ 



#include <stdbool.h>
#include <string.h>

#include "Internet/Internet.h"
#include "EthernetII.h"



int EthReadFrameHeader(uint8_t* buffer[], uint16_t* size, ETH_L2_FRAMEH_Type* ethFrame)
{
	if (*size < ETH_REG_FRAME_SIZE)
		return -1;
	
	uint8_t* buf = *buffer;
		
	ethFrame->DestMacAddr.BYTE[0] = buf[0];
	ethFrame->DestMacAddr.BYTE[1] = buf[1];
	ethFrame->DestMacAddr.BYTE[2] = buf[2];
	ethFrame->DestMacAddr.BYTE[3] = buf[3];
	ethFrame->DestMacAddr.BYTE[4] = buf[4];
	ethFrame->DestMacAddr.BYTE[5] = buf[5];
	
	ethFrame->SrcMacAddr.BYTE[0] = buf[6];
	ethFrame->SrcMacAddr.BYTE[1] = buf[7];
	ethFrame->SrcMacAddr.BYTE[2] = buf[8];
	ethFrame->SrcMacAddr.BYTE[3] = buf[9];
	ethFrame->SrcMacAddr.BYTE[4] = buf[10];
	ethFrame->SrcMacAddr.BYTE[5] = buf[11];
	
	ethFrame->EthType = (buf[12] << 8) | buf[13];

	
	if (ethFrame->EthType != VLAN_TPID)
	{
		ethFrame->VLAN.IsVLan = false;
		ethFrame->VLAN.TCI.hw = 0;

		*buffer	+= ETH_REG_FRAME_SIZE;
		*size	-= ETH_REG_FRAME_SIZE;
		return 0;
	}
		
	if (*size < ETH_VLAN_FRAME_SIZE)
		return -2;

	ethFrame->VLAN.IsVLan = true;
	ethFrame->VLAN.TCI.hw = (buf[14] << 8) | buf[15];

	ethFrame->EthType = (buf[16] << 8) | buf[17];


	*buffer	+= ETH_VLAN_FRAME_SIZE;
	*size	-= ETH_VLAN_FRAME_SIZE;

	return 1;
}


int EthWriteFrameHeader(uint8_t* buffer[], uint16_t* size, ETH_L2_FRAMEH_Type* ethFrame)
{
	if (!ethFrame->VLAN.IsVLan && (*size < ETH_REG_FRAME_SIZE))
		return -1;
	if (ethFrame->VLAN.IsVLan && (*size < ETH_VLAN_FRAME_SIZE))
		return -1;
	
	uint8_t* buf = *buffer;

	buf[0] = ethFrame->DestMacAddr.BYTE[0];
	buf[1] = ethFrame->DestMacAddr.BYTE[1];
	buf[2] = ethFrame->DestMacAddr.BYTE[2];
	buf[3] = ethFrame->DestMacAddr.BYTE[3];
	buf[4] = ethFrame->DestMacAddr.BYTE[4];
	buf[5] = ethFrame->DestMacAddr.BYTE[5];
	
	buf[6] = ethFrame->SrcMacAddr.BYTE[0];
	buf[7] = ethFrame->SrcMacAddr.BYTE[1];
	buf[8] = ethFrame->SrcMacAddr.BYTE[2];
	buf[9] = ethFrame->SrcMacAddr.BYTE[3];
	buf[10] = ethFrame->SrcMacAddr.BYTE[4];
	buf[11] = ethFrame->SrcMacAddr.BYTE[5];
	
	if (!ethFrame->VLAN.IsVLan)
	{
		buf[12] = (uint8_t)(ethFrame->EthType >> 8);
		buf[13] = (uint8_t)(ethFrame->EthType >> 0);
		
		*buffer += ETH_REG_FRAME_SIZE;
		*size   -= ETH_REG_FRAME_SIZE;
		
		return 0;
	}
	
	buf[12] = (uint8_t)(VLAN_TPID >> 8);
	buf[13] = (uint8_t)(VLAN_TPID >> 0);
	
	buf[14] = (uint8_t)(ethFrame->VLAN.TCI.hw >> 8);
	buf[15] = (uint8_t)(ethFrame->VLAN.TCI.hw >> 0);

	buf[16] = (uint8_t)(ethFrame->EthType >> 8);
	buf[17] = (uint8_t)(ethFrame->EthType >> 0);
	
	*buffer += ETH_VLAN_FRAME_SIZE;
	*size   -= ETH_VLAN_FRAME_SIZE;
	
	return 0;
}



int EthSetFrameHeader(INET_MAC_Type* destMacAddr, INET_MAC_Type* srcMacAddr, uint16_t ethType, ETH_VLAN_TAG_Type* vLanTag, ETH_L2_FRAMEH_Type* ethFrame )
{
	ethFrame->DestMacAddr	= *destMacAddr;
	ethFrame->SrcMacAddr	= *srcMacAddr;
	ethFrame->EthType		=  ethType;
	
	if ((vLanTag == 0) || (!vLanTag->IsVLan))
	{
		ethFrame->VLAN.IsVLan = false;
		ethFrame->VLAN.TCI.hw = 0;

		return 0;
	}
	
	ethFrame->VLAN = *vLanTag;
	
	return 0;
}


int EthSetVLanTag(uint16_t vid, uint16_t dei, uint16_t pcp, ETH_VLAN_TAG_Type* vLanTag)
{
	if (vid > 4095)
		return -1;
	if (dei > 1)
		return -2;
	if (pcp > 7)
		return -3;
	
	vLanTag->TCI.b.VID = vid;
	vLanTag->TCI.b.DEI = dei;
	vLanTag->TCI.b.PCP = pcp;

	vLanTag->IsVLan = (vid > 0);

	return 0;	
}



_Bool EthAreMacsEqual(INET_MAC_Type* mac1, INET_MAC_Type* mac2)
{
	uint8_t* mac1p = mac1->BYTE;
	uint8_t* mac2p = mac2->BYTE;
	
	uint32_t mac1r, mac2r;

	mac1r = (mac1p[0] << 0) | (mac1p[1] << 8);
	mac2r = (mac2p[0] << 0) | (mac2p[1] << 8);
	
	if (mac1r != mac2r)
		return false;

	mac1r = (mac1p[2] << 0) | (mac1p[3] << 8);
	mac2r = (mac2p[2] << 0) | (mac2p[3] << 8);
	
	if (mac1r != mac2r)
		return false;
		
	mac1r = (mac1p[4] << 0) | (mac1p[5] << 8);
	mac2r = (mac2p[4] << 0) | (mac2p[5] << 8);
		
	return (mac1r == mac2r);
}



_Bool EthIsBroadcast(INET_MAC_Type* mac)
{
	uint8_t* macp = mac->BYTE;

	if (((macp[0] << 0) | (macp[1] << 8)) != 0xffff)
		return false;
	
	if (((macp[2] << 0) | (macp[3] << 8)) != 0xffff)
		return false;
	
	return (((macp[4] << 0) | (macp[5] << 8)) == 0xffff);
}



_Bool EthIsMulticastIpV4(INET_MAC_Type* mac)
{
	uint8_t* macp = mac->BYTE;
		
	uint32_t macr = (macp[0] << 24) | (macp[1] << 16) | (macp[2] << 8) | macp[3];

	return ((macr & 0xffffff80) == 0x01005e00);
}



_Bool EthIsMulticastFromGroupIpV4(INET_MAC_Type* mac, INET_IP_Type* adr)
{
	if (!EthIsMulticastIpV4(mac))
		return false;
	
	uint8_t* macp = mac->BYTE;
	uint8_t* adrp = adr->BYTE;

	uint32_t macr = (macp[3] << 16) | (macp[4] << 8) | macp[5];

	uint32_t ipr = (adrp[1] << 16) | (adrp[2] << 8) | adrp[3];
	
	return ((macr ^ ipr) & 0x007fffff) == 0;
}


int EthMacFromMulticastGroupIpV4(INET_MAC_Type* mac, INET_IP_Type* adr)
{
	//ToDo	check ip mc range
	
	uint8_t* macp = mac->BYTE;
	uint8_t* adrp = adr->BYTE;
	
	macp[0] = 0x01;
	macp[1] = 0x00;
	macp[2] = 0x5e;
	macp[3] = adrp[1] & 0x7f;
	macp[4] = adrp[2];
	macp[5] = adrp[3];

	return 0;
}
