/*
 * EthernetII.h
 *
 * Created: 23.12.2018 10:57:22
 *  Author: Thomas
 */ 


#ifndef ETHERNETII_H_
#define ETHERNETII_H_


#include "misc.h"




#define ETH_REG_FRAME_SIZE		(14)
#define ETH_VLAN_FRAME_SIZE		(18)



#define VLAN_TPID	(0x8100u)



// VLan tag
typedef struct {
	_Bool IsVLan;					// IsVLan frame
	union {
		struct {
			uint16_t VID	:12;	// VLAN identifier
			uint16_t DEI	:1;		// Drop eligible indicator
			uint16_t PCP	:3;		// Priority code point
		} b;
		uint8_t  BYTE[2];
		uint16_t hw;
	} TCI;							// Tag control information
} ETH_VLAN_TAG_Type;


// EthernetII L2 frame header 
typedef struct {
	INET_MAC_Type		DestMacAddr;	// Destination MAC address
	INET_MAC_Type		SrcMacAddr;		// Source MAC address
	uint16_t			EthType;		// Ethernet type
	ETH_VLAN_TAG_Type	VLAN;			// VLan tag
} ETH_L2_FRAMEH_Type;


int EthReadFrameHeader(uint8_t* buffer[], uint16_t* size, ETH_L2_FRAMEH_Type* ethFrame);

int EthWriteFrameHeader(uint8_t* buffer[], uint16_t* size, ETH_L2_FRAMEH_Type* ethFrame);


int EthSetFrameHeader(	INET_MAC_Type*		destMacAddr,	// Destination MAC address
						INET_MAC_Type*		srcMacAddr,		// Source MAC address
						uint16_t			ethType,		// Ethernet type
						ETH_VLAN_TAG_Type*	vLanTag,		// VLan tag
						ETH_L2_FRAMEH_Type* ethFrame );		// EthFrameType;

int EthSetVLanTag(uint16_t vid, uint16_t dei, uint16_t pcp, ETH_VLAN_TAG_Type* vLanTag);


_Bool EthAreMacsEqual(INET_MAC_Type* mac1, INET_MAC_Type* mac2);


_Bool EthIsBroadcast(INET_MAC_Type* mac);


_Bool EthIsMulticastIpV4(INET_MAC_Type* mac);


_Bool EthIsMulticastFromGroupIpV4(INET_MAC_Type* mac, INET_IP_Type* adr);


int EthMacFromMulticastGroupIpV4(INET_MAC_Type* mac, INET_IP_Type* adr);



#endif /* ETHERNETII_H_ */