/*
 * ICMP.c
 *
 * Created: 29.12.2018 16:07:55
 *  Author: Thomas
 */ 


#include <stdbool.h>
#include <string.h>

#include "Internet/Internet.h"
#include "ICMP.h"



int	IcmpGetType(IPV4_HEADER_Type* ipHeader, uint8_t buffer[], uint16_t size)
{
	if (ipHeader->Protocol != ICMP_IPTYPE)
		return -1;
	if (size < ICMP_HEADER_SIZE)
		return -2;

	return buffer[0];
};



int IcmpReadEchoFrame(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader, ICMP_ECHO_FRAME_Type* echoFrame)
{
	if (ipHeader->Protocol != ICMP_IPTYPE)
		return -1;
	if (*size < ipHeader->PayloadLength)
		return -2;
	if (InetCalcChecksum(*buffer, ipHeader->PayloadLength) != 0x0000)
		return -3;

	uint8_t* buf = *buffer;
		
	echoFrame->IcmpHeader.Type		= buf[0];
	echoFrame->IcmpHeader.Code		= buf[1];
	echoFrame->IcmpHeader.Checksum	= (buf[2] << 8) | buf[3];
	echoFrame->Identifier			= (buf[4] << 8) | buf[5];
	echoFrame->SeqNumber			= (buf[6] << 8) | buf[7];
	
	if (echoFrame->IcmpHeader.Type != ICMP_TYPE_ECHO_REQ)
		if (echoFrame->IcmpHeader.Type != ICMP_TYPE_ECHO_REP)
			return -4;
	if (echoFrame->IcmpHeader.Code != 0)
		return -5;
	
	echoFrame->TotalLength = ipHeader->PayloadLength;
	echoFrame->Data.Length = echoFrame->TotalLength - ICMP_MIN_ECHO_FRAME_SIZE;
	if (echoFrame->Data.Length)
		echoFrame->Data.Buffer = buf + ICMP_MIN_ECHO_FRAME_SIZE;
	else
		echoFrame->Data.Buffer = 0;
	
	
	*buffer	+= ipHeader->PayloadLength;
	*size	-= ipHeader->PayloadLength;
	
	return 0;
}


int IcmpWriteEchoFrame(uint8_t* buffer[], uint16_t* size, ICMP_ECHO_FRAME_Type* echoFrame)
{
	if (*size < echoFrame->TotalLength)
		return -1;

	uint8_t* buf = *buffer;

	buf[0] = echoFrame->IcmpHeader.Type;
	buf[1] = echoFrame->IcmpHeader.Code;
	buf[2] = 0; // Checksum clear
	buf[3] = 0; // Checksum clear
	buf[4] = (uint8_t)(echoFrame->Identifier >> 8);
	buf[5] = (uint8_t)(echoFrame->Identifier >> 0);
	buf[6] = (uint8_t)(echoFrame->SeqNumber >> 8);
	buf[7] = (uint8_t)(echoFrame->SeqNumber >> 0);


	if (echoFrame->Data.Length)
		memcpy(&buf[ICMP_MIN_ECHO_FRAME_SIZE], echoFrame->Data.Buffer, echoFrame->Data.Length);
		
	
	echoFrame->IcmpHeader.Checksum = InetCalcChecksum(buf, echoFrame->TotalLength);

	buf[2] = (uint8_t)(echoFrame->IcmpHeader.Checksum >> 8);
	buf[3] = (uint8_t)(echoFrame->IcmpHeader.Checksum >> 0);
	

	*buffer	+= echoFrame->TotalLength;
	*size	-= echoFrame->TotalLength;
	
	return 0;
}


int IcmpSetEchoFrame(uint8_t echoType, uint16_t identifier, uint16_t seqNumber, ICMP_PTR_DATA_Type* data, ICMP_ECHO_FRAME_Type* echoFrame)
{
	if (echoType != ICMP_TYPE_ECHO_REQ)
		if (echoType != ICMP_TYPE_ECHO_REP)
			return -1;
	
	echoFrame->IcmpHeader.Type		= echoType;
	echoFrame->IcmpHeader.Code		= 0;
	echoFrame->IcmpHeader.Checksum	= 0;
	
	echoFrame->Identifier	= identifier;
	echoFrame->SeqNumber	= seqNumber;
	
	uint16_t totalLength = ICMP_MIN_ECHO_FRAME_SIZE;

	if (data)
	{
		echoFrame->Data = *data;
		totalLength += echoFrame->Data.Length;
	}
	
	echoFrame->TotalLength = totalLength;
	
	return 0;
}
