/*
 * ICMP.h
 *
 * Created: 29.12.2018 16:08:05
 *  Author: Thomas
 */ 


#ifndef ICMP_H_
#define ICMP_H_


#include "Internet/Internet.h"
#include "Internet/IP/IPV4.h"


#define ICMP_IPTYPE					1	// IP type

#define	ICMP_HEADER_SIZE			4	// ICMP header size
#define	ICMP_MIN_ECHO_FRAME_SIZE	8	// ICMP echo size


#define ICMP_TYPE_ECHO_REP			0	// Echo Reply
#define ICMP_TYPE_DEST_UNREACH		3	// Destination Unreachable
#define ICMP_TYPE_REDIRECT			5	// Redirect (change route)
#define ICMP_TYPE_ECHO_REQ			8	// Echo Request
#define ICMP_TYPE_9					9	// TBD
#define ICMP_TYPE_10				10	// TBD
#define ICMP_TYPE_TIME_EXCEEDED		11	// Time Exceeded
#define ICMP_TYPE_PARAMETERPROB		12	// Parameter Problem
#define ICMP_TYPE_TIMESTAMP			13	// Timestamp Request
#define ICMP_TYPE_TIMESTAMPREPLY	14	// Timestamp Reply
#define ICMP_TYPE_40				40	// RFC 2522


// Codes for Destination Unreachable
#define ICMP_NR_UNREACH				15

#define ICMP_UNR_NET_UNREACH		0	// Network Unreachable
#define ICMP_UNR_HOST_UNREACH		1	// Host Unreachable
#define ICMP_UNR_PROT_UNREACH		2	// Protocol Unreachable
#define ICMP_UNR_PORT_UNREACH		3	// Port Unreachable
#define ICMP_UNR_FRAG_NEEDED		4	// Fragmentation Needed/DF set
#define ICMP_UNR_SR_FAILED			5	// Source Route failed
#define ICMP_UNR_NET_UNKNOWN		6
#define ICMP_UNR_HOST_UNKNOWN		7
#define ICMP_UNR_HOST_ISOLATED		8
#define ICMP_UNR_NET_ANO			9
#define ICMP_UNR_HOST_ANO			10
#define ICMP_UNR_NET_UNR_TOS		11
#define ICMP_UNR_HOST_UNR_TOS		12
#define ICMP_UNR_PKT_FILTERED		13	// Packet filtered
#define ICMP_UNR_PREC_VIOLATION		14	// Precedence violation
#define ICMP_UNR_PREC_CUTOFF		15	// Precedence cut off


// Codes for TIME_EXCEEDED
#define ICMP_TEXC_TTL				0	// TTL count exceeded


typedef struct {
	uint8_t*	Buffer;
	uint16_t	Length;
} ICMP_PTR_DATA_Type;


// ICMP Header
typedef struct {
	uint8_t		Type;
	uint8_t		Code;
	uint16_t	Checksum;
} ICMP_HEADER_Type;



// ICMP Echo
typedef struct {
	ICMP_HEADER_Type	IcmpHeader;
	uint16_t			Identifier;
	uint16_t			SeqNumber;
	uint16_t			TotalLength;
	ICMP_PTR_DATA_Type	Data;
} ICMP_ECHO_FRAME_Type;


int	IcmpGetType(IPV4_HEADER_Type* ipHeader, uint8_t buffer[], uint16_t size);

int IcmpReadEchoFrame(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader, ICMP_ECHO_FRAME_Type* echoFrame);

int IcmpWriteEchoFrame(uint8_t* buffer[], uint16_t* size, ICMP_ECHO_FRAME_Type* echoFrame);

int IcmpSetEchoFrame(uint8_t echoType, uint16_t identifier, uint16_t seqNumber, ICMP_PTR_DATA_Type* data, ICMP_ECHO_FRAME_Type* echoFrame);




#endif /* ICMP_H_ */