/*
 * IGMP.h
 *
 * Created: 31.12.2018 19:23:21
 *  Author: Thomas
 */ 


#ifndef IGMP_H_
#define IGMP_H_


#include "Internet/Internet.h"
#include "Internet/IP/IPV4.h"


#define IGMP_IPTYPE					2	// IP type

#define	IGMP_HEADER_SIZE_V1			8	// IGMP header size
#define	IGMP_HEADER_SIZE_V2			8	// IGMP header size

#define IGMP_IP_QUERY			((INET_IP_Type){{ 224, 0, 0, 1}})
#define IGMP_IP_REPORT_V3		((INET_IP_Type){{ 224, 0, 0, 22}})

#define IGMP_MAC_QUERY			((INET_MAC_Type){{0x01, 0x00, 0x5e, 0x00, 0x00, 0x01}})
#define IGMP_MAC_REPORT_V3		((INET_MAC_Type){{0x01, 0x00, 0x5e, 0x00, 0x00, 0x16}})


	



typedef struct {
	uint8_t			Type;
	uint8_t			Code;
	uint16_t		Checksum;
	INET_IP_Type	GroupAddr;
	uint16_t		Length;
	uint8_t			IgmpVersion;
} IGMP_FRAME_Type;







// Message types, including version number.
#define IGMP_MEMBERSHIP_QUERY		0x11		// Membership query
#define IGMP_V1_MEMBERSHIP_REPORT	0x12		// Version 1 membership report
#define IGMP_V2_MEMBERSHIP_REPORT	0x16		// Version 2 membership report
#define IGMP_V2_LEAVE_GROUP			0x17		// Leave-group message

#define IGMP_DVMRP					0x13		// DVMRP routing message
#define IGMP_PIM					0x14		// PIM routing message
#define IGMP_TRACE					0x15

#define IGMP_MTRACE_RESP			0x1e		// Traceroute resp.(to sender)
#define IGMP_MTRACE					0x1f		// Multicast traceroute messages

#define IGMP_V3_MEMBERSHIP_REPORT	0x22		// Version 3 membership report


#define IGMP_MAX_HOST_REPORT_DELAY	10			// Max delay for response to query (in seconds) according to RFC1112
#define IGMP_TIMER_SCALE			10			// Denotes that the igmp code field specifies time in 10th of seconds

//// States for the IGMP v2 state table.
//#define IGMP_DELAYING_MEMBER		1
//#define IGMP_IDLE_MEMBER			2
//#define IGMP_LAZY_MEMBER			3
//#define IGMP_SLEEPING_MEMBER		4
//#define IGMP_AWAKENING_MEMBER		5
//
//// States for IGMP router version cache.
//#define IGMP_v1_ROUTER				1
//#define IGMP_v2_ROUTER				2



int IgmpReadFrameV1(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader, IGMP_FRAME_Type* igmpFrame);

int IgmpWriteFrameV1(uint8_t* buffer[], uint16_t* size, IGMP_FRAME_Type* igmpFrame);

int IgmpSetQueryFrameV1(IGMP_FRAME_Type* igmpFrame);

int IgmpSetReportFrameV1(INET_IP_Type* groupAddr, IGMP_FRAME_Type* igmpFrame);



int IgmpReadFrameV2(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader, IGMP_FRAME_Type* igmpFrame);

int IgmpWriteFrameV2(uint8_t* buffer[], uint16_t* size, IGMP_FRAME_Type* igmpFrame);

int IgmpSetQueryFrameV2(uint8_t maxRespTime, IGMP_FRAME_Type* igmpFrame);

int IgmpSetReportFrameV2(INET_IP_Type* groupAddr, IGMP_FRAME_Type* igmpFrame);


int IgmpReadFrame(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader, IGMP_FRAME_Type* igmpFrame);

int IgmpWriteFrame(uint8_t* buffer[], uint16_t* size, IGMP_FRAME_Type* igmpFrame);


#endif /* IGMP_H_ */