/*
 * IPV4.h
 *
 * Created: 27.12.2018 15:23:57
 *  Author: Thomas
 */ 


#ifndef IPV4_H_
#define IPV4_H_



#define	IPV4_ETHERTYPE			(0x0800)	// IPV4 "EtherType"
#define	IPV4_MIN_HEADER_SIZE	(20)		// IPV4 header size
#define	IPV4_VERSION			(4)			// IPV4 version number
#define	IPV4_DEFTTL				(64)		// Default TTL



typedef struct {
	uint8_t*	Buffer;
	uint16_t	Length;
} IPV4_PTR_OPTION_Type;


IPV4_PTR_OPTION_Type* IpV4GetRouterAlertOptionPtr(void);



typedef struct {
	uint8_t					Version;				// 4 Bit LSB
	uint8_t					HeaderLengthX32B;		// 4 Bit MSB
	union {
		struct {
			uint8_t			ECN		:2;
			uint8_t			DSCP	:6;
		} DS;
		struct {
			uint8_t							:1;
			uint8_t			MinCost			:1;
			uint8_t			Reliability		:1;
			uint8_t			Throughput		:1;
			uint8_t			Lowdelay		:1;
			uint8_t			IpPrecedence	:3;
		} TOS;
		uint8_t BYTE[1];
		uint8_t by;
	}TOS;
    uint16_t				TotalLength;
    uint16_t				Identification;
	union {
		struct {
			uint16_t		Offset	:13;
			uint16_t		More	:1;
			uint16_t		Dont	:1;
			uint16_t				:1;
		} b;
		uint8_t  BYTE[2];
		uint16_t hw;
	} FRAG;
    uint8_t					TimeToLive;
    uint8_t					Protocol;
    uint16_t				Checksum;
    INET_IP_Type			SrcIpAddr;
    INET_IP_Type			DestIpAddr;
	IPV4_PTR_OPTION_Type	Option;
	uint16_t				PayloadLength;
} IPV4_HEADER_Type;


#define IPV4_HEADER_DS_ECN_Pos						0			// Position "DS.ECN"
#define IPV4_HEADER_DS_ECN_Msk						(_U_(0x3) << IPV4_HEADER_DS_ECN_Pos)

#define	IPV4_HEADER_DS_ECN_NOT_ECT_Val				_U_(0x0)	
#define	IPV4_HEADER_DS_ECN_ECT1_Val					_U_(0x1)	
#define	IPV4_HEADER_DS_ECN_ECT0_Val					_U_(0x2)	
#define	IPV4_HEADER_DS_ECN_CE_Val					_U_(0x3)	

#define	IPV4_HEADER_DS_ECN(value)					(((value) << IPV4_HEADER_DS_ECN_Pos) & IPV4_HEADER_DS_ECN_Msk)	

#define IPV4_HEADER_DS_ECN_NOT_ECT					(IPV4_HEADER_DS_ECN_NOT_ECT_Val	<< IPV4_HEADER_DS_ECN_Pos)
#define IPV4_HEADER_DS_ECN_ECT1						(IPV4_HEADER_DS_ECN_ECT1_Val	<< IPV4_HEADER_DS_ECN_Pos)
#define IPV4_HEADER_DS_ECN_ECT0						(IPV4_HEADER_DS_ECN_ECT0_Val	<< IPV4_HEADER_DS_ECN_Pos)
#define IPV4_HEADER_DS_ECN_CE						(IPV4_HEADER_DS_ECN_CE_Val		<< IPV4_HEADER_DS_ECN_Pos)



#define IPV4_HEADER_DS_DSCP_Pos						0			// Position "DS.DSCP"
#define IPV4_HEADER_DS_DSCP_Msk						(_U_(0x3f) << IPV4_HEADER_DS_DSCP_Pos)

#define	IPV4_HEADER_DS_DSCP_AF11_Val				_U_(0x0a)
#define	IPV4_HEADER_DS_DSCP_AF12_Val				_U_(0x0c)
#define	IPV4_HEADER_DS_DSCP_AF13_Val				_U_(0x0e)
#define	IPV4_HEADER_DS_DSCP_AF21_Val				_U_(0x12)
#define	IPV4_HEADER_DS_DSCP_AF22_Val				_U_(0x14)
#define	IPV4_HEADER_DS_DSCP_AF23_Val				_U_(0x16)
#define	IPV4_HEADER_DS_DSCP_AF31_Val				_U_(0x1a)
#define	IPV4_HEADER_DS_DSCP_AF32_Val				_U_(0x1c)
#define	IPV4_HEADER_DS_DSCP_AF33_Val				_U_(0x1e)
#define	IPV4_HEADER_DS_DSCP_AF41_Val				_U_(0x22)
#define	IPV4_HEADER_DS_DSCP_AF42_Val				_U_(0x24)
#define	IPV4_HEADER_DS_DSCP_AF43_Val				_U_(0x26)
#define	IPV4_HEADER_DS_DSCP_EF_Val					_U_(0x2e)

#define	IPV4_HEADER_DS_DSCP(value)					(((value) << IPV4_HEADER_DS_DSCP_Pos) & IPV4_HEADER_DS_DSCP_Msk)

#define IPV4_HEADER_DS_DSCP_AF11					(IPV4_HEADER_DS_DSCP_AF11_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF12					(IPV4_HEADER_DS_DSCP_AF12_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF13					(IPV4_HEADER_DS_DSCP_AF13_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF21					(IPV4_HEADER_DS_DSCP_AF21_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF22					(IPV4_HEADER_DS_DSCP_AF22_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF23					(IPV4_HEADER_DS_DSCP_AF23_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF31					(IPV4_HEADER_DS_DSCP_AF31_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF32					(IPV4_HEADER_DS_DSCP_AF32_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF33					(IPV4_HEADER_DS_DSCP_AF33_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF41					(IPV4_HEADER_DS_DSCP_AF41_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF42					(IPV4_HEADER_DS_DSCP_AF42_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_AF43					(IPV4_HEADER_DS_DSCP_AF43_Val	<< IPV4_HEADER_DS_DSCP_Pos)
#define IPV4_HEADER_DS_DSCP_EF						(IPV4_HEADER_DS_DSCP_EF_Val		<< IPV4_HEADER_DS_DSCP_Pos)


#define IPV4_HEADER_TOS_MINCOST_Pos					1			// Position "TOS.MINCOST"
#define IPV4_HEADER_TOS_MINCOST						(_U_(0x1) << IPV4_HEADER_TOS_MINCOST_Pos)
#define IPV4_HEADER_TOS_LOWCOST						IPV4_HEADER_TOS_MINCOST

#define IPV4_HEADER_TOS_RELIABILITY_Pos				2			// Position "TOS.RELIABILITY"
#define IPV4_HEADER_TOS_RELIABILITY					(_U_(0x1) << IPV4_HEADER_TOS_RELIABILITY_Pos)

#define IPV4_HEADER_TOS_THROUGHPUT_Pos				3			// Position "TOS.THROUGHPUT"
#define IPV4_HEADER_TOS_THROUGHPUT					(_U_(0x1) << IPV4_HEADER_TOS_THROUGHPUT_Pos)

#define IPV4_HEADER_TOS_LOWDELAY_Pos				4			// Position "TOS.LOWDELAY"
#define IPV4_HEADER_TOS_LOWDELAY					(_U_(0x1) << IPV4_HEADER_TOS_LOWDELAY_Pos)


#define IPV4_HEADER_TOS_IPPREC_Pos					5			// Position "TOS.IpPrecedence"
#define IPV4_HEADER_TOS_IPPREC_Msk					(_U_(0x7) << IPV4_HEADER_TOS_IPPREC_Pos)

#define	IPV4_HEADER_TOS_IPPREC_ROUTINE_Val			_U_(0x0)
#define	IPV4_HEADER_TOS_IPPREC_PRIORITY_Val			_U_(0x1)
#define	IPV4_HEADER_TOS_IPPREC_IMMEDIATE_Val		_U_(0x2)
#define	IPV4_HEADER_TOS_IPPREC_FLASH_Val			_U_(0x3)
#define	IPV4_HEADER_TOS_IPPREC_FLASHOVERRIDE_Val	_U_(0x4)
#define	IPV4_HEADER_TOS_IPPREC_CRITIC_ECP_Val		_U_(0x5)
#define	IPV4_HEADER_TOS_IPPREC_INTERNETCONTROL_Val	_U_(0x6)
#define	IPV4_HEADER_TOS_IPPREC_NETCONTROL_Val		_U_(0x7)

#define	IPV4_HEADER_TOS_IPPREC(value)				(((value) << IPV4_HEADER_TOS_IPPREC_Pos) & IPV4_HEADER_TOS_IPPREC_Msk)

#define IPV4_HEADER_TOS_IPPREC_ROUTINE				(IPV4_HEADER_TOS_IPPREC_ROUTINE_Val			<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_PRIORITY				(IPV4_HEADER_TOS_IPPREC_PRIORITY_Val		<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_IMMEDIATE			(IPV4_HEADER_TOS_IPPREC_IMMEDIATE_Val		<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_FLASH				(IPV4_HEADER_TOS_IPPREC_FLASH_Val			<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_FLASHOVERRIDE		(IPV4_HEADER_TOS_IPPREC_FLASHOVERRIDE_Val	<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_CRITIC_ECP			(IPV4_HEADER_TOS_IPPREC_CRITIC_ECP_Val		<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_INTERNETCONTROL		(IPV4_HEADER_TOS_IPPREC_INTERNETCONTROL_Val	<< IPV4_HEADER_TOS_IPPREC_Pos)
#define IPV4_HEADER_TOS_IPPREC_NETCONTROL			(IPV4_HEADER_TOS_IPPREC_NETCONTROL_Val		<< IPV4_HEADER_TOS_IPPREC_Pos)



#define IPV4_HEADER_FRAG_OFFSET_Pos					0			// Position "FRAG.Offset"
#define IPV4_HEADER_FRAG_OFFSET_Msk					(_U_(0x1fff) << IPV4_HEADER_FRAG_OFFSET_Pos)

#define	IPV4_HEADER_FRAG_OFFSET(value)				(((value) << IPV4_HEADER_FRAG_OFFSET_Pos) & IPV4_HEADER_FRAG_OFFSET_Msk)


#define IPV4_HEADER_FRAG_MORE_Pos					13			// Position "FRAG.MORE"
#define IPV4_HEADER_FRAG_MORE						(_U_(0x1) << IPV4_HEADER_FRAG_MORE_Pos)


#define IPV4_HEADER_FRAG_DONOT_Pos					14			// Position "FRAG.DONOT"
#define IPV4_HEADER_FRAG_DONOT						(_U_(0x1) << IPV4_HEADER_FRAG_DONOT_Pos)




int IpV4ReadFrameHeader(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader);

int IpV4WriteFrameHeader(uint8_t* buffer[], uint16_t* size, IPV4_HEADER_Type* ipHeader);

int IpV4SetFrameHeader(	uint8_t					protocol,
						INET_IP_Type*			srcIpAddr,
						INET_IP_Type*			destIpAddr,
						uint16_t				payloadLength,
						IPV4_PTR_OPTION_Type*	option,
						IPV4_HEADER_Type*		ipHeader );


uint32_t IpV4CalcChecksumPseudoHeader(IPV4_HEADER_Type* ipHeader, uint16_t protoLength);


_Bool IpV4IpEqual(INET_IP_Type* ip1, INET_IP_Type* ip2);

_Bool IpV4IpInSameSubnet(INET_IP_Type* ip1, INET_IP_Type* ip2, INET_IP_Type* snm);

_Bool IpV4IsAny(INET_IP_Type* ip);


#endif /* IPV4_H_ */