/*
 * Socket.h
 *
 * Created: 17.11.2018 14:55:22
 *  Author: Thomas
 */ 


#ifndef SOCKET_H_
#define SOCKET_H_


#include "Internet\Internet.h"


#define SOCKET_NOERR_ASYNCH_RUNNING	(1)
#define SOCKET_NOERR				(0)
#define SOCKET_ERROR_GENERAL		(-100)
#define SOCKET_ERROR_PARAMETER1		(-101)
#define SOCKET_ERROR_PARAMETER2		(-102)
#define SOCKET_ERROR_PARAMETER3		(-103)
#define SOCKET_ERROR_PARAMETER4		(-104)
#define SOCKET_ERROR_PARAMETER5		(-105)
#define SOCKET_ERROR_CHANNEL_IN_USE	(-106)
#define SOCKET_ERROR_TIMEOUT		(-107)
#define SOCKET_ERROR_RBUFFER_SIZE	(-108)
#define SOCKET_ERROR_TBUFFER_SIZE	(-109)
#define SOCKET_ERROR_ASYNCH_BUSY	(-110)



#define  SO_TIME_INFINITE	(-1)



typedef enum {
	SocketTypeNothing = -1,
	//SocketTypeTcp,
	SocketTypeUdp,
	SocketTypeIp,
	SocketTypeMacRaw,
} SocketType_Type;


typedef enum {
	SocketIpProtocolNothing = -1,
	//SocketIpProtocoIp = WNET_SRB_SIPPR_IPPROTO_IP_Val,
	//SocketIpProtocoIcmp = WNET_SRB_SIPPR_IPPROTO_ICMP_Val,
	//SocketIpProtocoIgmp = WNET_SRB_SIPPR_IPPROTO_IGMP_Val,
	//SocketIpProtocoGgp = WNET_SRB_SIPPR_IPPROTO_GGP_Val,
	//SocketIpProtocoTcp = WNET_SRB_SIPPR_IPPROTO_TCP_Val,
	//SocketIpProtocoPup = WNET_SRB_SIPPR_IPPROTO_PUP_Val,
	//SocketIpProtocoUdp = WNET_SRB_SIPPR_IPPROTO_UDP_Val,
	//SocketIpProtocoIdp = WNET_SRB_SIPPR_IPPROTO_IDP_Val,
	//SocketIpProtocoNd = WNET_SRB_SIPPR_IPPROTO_ND_Val,
	//SocketIpProtocoRaw = WNET_SRB_SIPPR_IPPROTO_RAW_Val,
} SocketIpProtocol_Type;


typedef enum {
	SocketStatusNothing			= -1,
	SocketStatusClosed			= WNET_SRB_SSR_STAT_SOCK_CLOSED_Val,
	SocketStatusInit			= WNET_SRB_SSR_STAT_SOCK_INIT_Val,
	SocketStatusListen			= WNET_SRB_SSR_STAT_SOCK_LISTEN_Val,
	SocketStatusEstablished		= WNET_SRB_SSR_STAT_SOCK_ESTABLISHED_Val,
	SocketStatusCloseWait		= WNET_SRB_SSR_STAT_SOCK_CLOSE_WAIT_Val,
	SocketStatusUdp				= WNET_SRB_SSR_STAT_SOCK_UDP_Val,
	SocketStatusIp				= WNET_SRB_SSR_STAT_SOCK_IPRAW_Val,
	SocketStatusMac				= WNET_SRB_SSR_STAT_SOCK_MACRAW_Val,
	SocketStatusSynchSent		= WNET_SRB_SSR_STAT_SOCK_SYNSENT_Val,
	SocketStatusSynchReceive	= WNET_SRB_SSR_STAT_SOCK_SYNRECV_Val,
	SocketStatusFinWait			= WNET_SRB_SSR_STAT_SOCK_FIN_WAIT_Val,
	SocketStatusClosing			= WNET_SRB_SSR_STAT_SOCK_CLOSING_Val,
	SocketStatusTimeWait		= WNET_SRB_SSR_STAT_SOCK_TIME_WAIT_Val,
	SocketStatusLastAck			= WNET_SRB_SSR_STAT_SOCK_LAST_ACK_Val,
} SocketStatus_Type;



typedef struct {
	int							Channel;
	SocketType_Type				Type;
	SocketIpProtocol_Type		IpProtocol;
	SocketStatus_Type			Status;
} Socket_Type;


typedef struct {
	INET_IP_Type		Ip;
	uint16_t			Port;
} SocketEndPoint_Type;


typedef struct {
	SocketEndPoint_Type SOURCE;
	uint16_t			DataLength;
} SocketUdpHeader_Type;



int SocketOpen(	Socket_Type*			socket,
				uint8_t					channel,
				SocketType_Type			socketType,
				SocketIpProtocol_Type	socketIpProtocol,
				uint16_t				port,
				int						timeOut_ms );		// -1 == SO_TIME_INFINITE

int SocketClose(Socket_Type* socket);



int GetSxReceivedSize(uint8_t controlPhaseBsel, uint16_t* rxSize);

int GetSxTxBufferFreeSize(uint8_t controlPhaseBsel, uint16_t* txFreeSize);

int WaitUntilSxLastSendFinished(uint8_t controlPhaseBsel);



SocketUdpHeader_Type SocketGetReceiveUdpHeader(void);

typedef struct {
	_Bool			IsReady;
	_Bool			IsBusy;
	_Bool			IsCompleted;
	int				AsyncState;
	int				CallBackState;
	uint16_t		ReadWriteBytes;
	Socket_Type*	Socket;
} SOCKET_RW_ASYNC_RESULT_Type;



int SocketGetAsyncResult(SOCKET_RW_ASYNC_RESULT_Type* asyncResult);

typedef int (*SocketAsyncCallBack_Type)(SOCKET_RW_ASYNC_RESULT_Type* asyncResult);



int SocketReceiveUdp(		Socket_Type* socket,
							uint8_t* buffer,
							uint16_t bufferSize,
							uint16_t* receivedBytes);

int SocketBeginReceiveUdp(	Socket_Type* socket,
							uint8_t* buffer,
							uint16_t bufferSize,
							SocketAsyncCallBack_Type callBack);


	
int SocketSendTo(		Socket_Type* socket,
						uint8_t* buffer,
						uint16_t count,
						SocketEndPoint_Type* endPoint);

int SocketBeginSendTo(	Socket_Type* socket,
						uint8_t* buffer,
						uint16_t count,
						SocketEndPoint_Type* endPoint,
						SocketAsyncCallBack_Type callBack);



int SocketReceiveMacRaw(		Socket_Type* socket,
								uint8_t* buffer,
								uint16_t bufferSize, 
								uint16_t* receivedBytes);

int SocketBeginReceiveMacRaw(	Socket_Type* socket,
								uint8_t* buffer,
								uint16_t bufferSize,
								SocketAsyncCallBack_Type callBack);



int SocketSendMacRaw(		Socket_Type* socket,
							uint8_t* buffer,
							uint16_t count);

int SocketBeginSendMacRaw(	Socket_Type* socket,
							uint8_t* buffer,
							uint16_t count,
							SocketAsyncCallBack_Type callBack);



#endif /* SOCKET_H_ */