/*
 * SocketMain.c
 *
 * Created: 16.12.2018 18:25:15
 *  Author: Thomas
 */ 


#include <stdbool.h>
#include <string.h>

#include "sam.h"

#include "misc.h"
#include "IO.h"
#include "Time/Time.h"

#include "Internet/Internet.h"
#include "Internet/EthernetII/EthernetII.h"
#include "Internet/ARP/ARP.h"
#include "Internet/IP/IPV4.h"
#include "Internet/ICMP/ICMP.h"
#include "Internet/IGMP/IGMP.h"
#include "Internet/UDP/UDP.h"
#include "Socket/Socket.h"


#include "Router/Router.h"

#include "SocketMain.h"

#include "Time/Time.h"


#define CHECK_RETV_RET(x)	do { int ret_value_func = (x); if (ret_value_func < 0) { return; } } while(0);
#define CHECK_RETV_RETV(x)	do { int ret_value_func = (x); if (ret_value_func < 0) { return (ret_value_func); } } while(0);



static Socket_Type MacRawSocket;

static uint8_t RxBuffer[1519];	// 6(DestMac) + 6(srcMac) + 4(VlanTAG) + 2(Type) + 1500(Payload) + 1(0x00 String termination, convert payload to string in receive frame)
static uint8_t TxBuffer[1518];	// 6(DestMac) + 6(srcMac) + 4(VlanTAG) + 2(Type) + 1500(Payload)


// ----------------------------------------------------------------------------------------------


int SocketMainInit(void)
{

	#ifdef ETH_10MFD
		// PHY Reset / 10MBit full duplex
		CHECK_RETV_RETV(WNetWriteByte(WNET_CRB_PHYCFGR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, 0));
	
		uint8_t phyCfg =  WNET_CRB_PHYCFGR_RST
						| WNET_CRB_PHYCFGR_OPMDC_10FDAND
						| WNET_CRB_PHYCFGR_OPMD_OPMDC;
	
		CHECK_RETV_RETV(WNetWriteByte(WNET_CRB_PHYCFGR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, phyCfg));
	#endif	
		
	
	// MAC
	CHECK_RETV_RETV(WNetWriteMacType(WNET_CRB_SHAR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, &MY_MAC));


	WNET_SOCKET_INIT_Type si = {
		WNET_SRB_RXBUF_SIZE_16KB, WNET_SRB_TXBUF_SIZE_16KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB,
		WNET_SRB_RXBUF_SIZE_0KB, WNET_SRB_TXBUF_SIZE_0KB, };
	CHECK_RETV_RETV(WNetSocketInit(&si));


	// Socket Interrupt
	WNET_CRB_SIMR_Type simr =  {.b = { .S0 = true } };
	CHECK_RETV_RETV(WNetWriteByte(WNET_CRB_SIMR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, simr.by));

	WNET_SRB_IMR_Type imr = {.b = { .RECV = true } };
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_IMR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S0REG, imr.by));
	
	PORTB->DIRCLR.reg = WN_IRQ_A3_PB09;
	PORTB->OUTSET.reg = WN_IRQ_A3_PB09;	// Pull-up
	PORTB->PINCFG[WN_IRQ_A3_PB09_Pos].bit.INEN = true;
	PORTB->PINCFG[WN_IRQ_A3_PB09_Pos].bit.PULLEN = true;


	CHECK_RETV_RETV(SocketOpen(&MacRawSocket, 0, SocketTypeMacRaw, SocketIpProtocolNothing, 0, SO_TIME_INFINITE));
	
	
	return SOCKET_NOERR;
}


// ----------------------------------------------------------------------------------------------


static int ReceiveSocket0IsCompleted(SOCKET_RW_ASYNC_RESULT_Type* asyncResult)
{
	return RouterReceiveMacFrame(&MacRawSocket, RxBuffer, sizeof(RxBuffer), asyncResult->ReadWriteBytes, TxBuffer, sizeof(TxBuffer));
}


// ----------------------------------------------------------------------------------------------


int SocketMain(void)
{
	SOCKET_RW_ASYNC_RESULT_Type soAsyncResult;
		
	CHECK_RETV_RETV(SocketGetAsyncResult(&soAsyncResult));
	
	CHECK_RETV_RETV(soAsyncResult.AsyncState);
	CHECK_RETV_RETV(soAsyncResult.CallBackState);

		
	if(soAsyncResult.IsReady)
	{
		if (!(PORTB->IN.reg & WN_IRQ_A3_PB09))
		{
			WNET_CRB_SIR_Type csir = { 0 };

			CHECK_RETV_RETV(WNetReadByte(WNET_CRB_SIR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, &csir.by));
				
				
			if (csir.b.S0)
			{
				WNET_SRB_SIR_Type ssir = {.b = { .RECV = true }};
				CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_SIR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S0REG, ssir.by));
					
				CHECK_RETV_RETV(SocketBeginReceiveMacRaw(&MacRawSocket, RxBuffer, sizeof(RxBuffer), ReceiveSocket0IsCompleted));
			}
			else
			{
				;
			}
		}
		else
		{
			CHECK_RETV_RETV(RouterProcessCyclicTasks(&MacRawSocket, TxBuffer, sizeof(TxBuffer)));
		}
	}
	
	return SOCKET_NOERR;
}


// ----------------------------------------------------------------------------------------------


int SendGratuitousArp(Socket_Type* socket, uint8_t txBuffer[], uint16_t txBufferSize, ETH_VLAN_TAG_Type* vLanTag, INET_IP_Type* ip)
{
	uint8_t* pTxBuf		= txBuffer;
	uint16_t sizeTxBuf	= txBufferSize;
	
	
	ETH_L2_FRAMEH_Type ethFrameTx;
	CHECK_RETV_RETV(EthSetFrameHeader(&MAC_FF, &MY_MAC, ARP_ETHERTYPE, vLanTag, &ethFrameTx));
		
	ARP_FRAME_Type arpFrameTx;
	CHECK_RETV_RETV(ArpSetFrame(	ARP_OPERATION_REQ,
									&MY_MAC,
									ip,
									&MAC_00,
									ip,
									&arpFrameTx ));
	
	
	CHECK_RETV_RETV(EthWriteFrameHeader(&pTxBuf, &sizeTxBuf, &ethFrameTx));
		
	CHECK_RETV_RETV(ArpWriteFrame(&pTxBuf, &sizeTxBuf, &arpFrameTx));
	
	CHECK_RETV_RETV(SocketSendMacRaw(socket, txBuffer, pTxBuf - txBuffer));
	
	return 0;
}


// ----------------------------------------------------------------------------------------------


int SendDummyArp(Socket_Type* socket, uint8_t txBuffer[], uint16_t txBufferSize, ETH_VLAN_TAG_Type* vLanTag, INET_IP_Type* ip)
{
	uint8_t* pTxBuf		= txBuffer;
	uint16_t sizeTxBuf	= txBufferSize;
	
	
	ETH_L2_FRAMEH_Type ethFrameTx;
	CHECK_RETV_RETV(EthSetFrameHeader(&MAC_FF, &MY_MAC, ARP_ETHERTYPE, vLanTag, &ethFrameTx));
	
	ARP_FRAME_Type arpFrameTx;
	CHECK_RETV_RETV(ArpSetFrame(	ARP_OPERATION_REQ,
									&MY_MAC,
									&IP_ANY,
									&MAC_00,
									ip,
									&arpFrameTx ));
	
	
	CHECK_RETV_RETV(EthWriteFrameHeader(&pTxBuf, &sizeTxBuf, &ethFrameTx));
	
	CHECK_RETV_RETV(ArpWriteFrame(&pTxBuf, &sizeTxBuf, &arpFrameTx));
	
	CHECK_RETV_RETV(SocketSendMacRaw(socket, txBuffer, pTxBuf - txBuffer));
	
	return 0;
}
	
	
// ----------------------------------------------------------------------------------------------