/*
 * StatusLed.c
 *
 * Created: 09.02.2019 18:18:23
 *  Author: Thomas
 */ 

#include <stdbool.h>
#include "sam.h"

#include "IO.h"
#include "clocks.h"
#include "misc.h"
#include "Time/Time.h"
#include "StatusLed.h"



// ----------------------------------------------------------------------------------------------


void StatusLedInit(void)
{
	// Enable module clock for TC
	MCLK->APBBMASK.bit.TC3_ = true;
		
	// Turn on clock for TC core
	GCLK->PCHCTRL[TC3_GCLK_ID].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_192M)
								   | GCLK_PCHCTRL_CHEN;
		
	while(GCLK->PCHCTRL[TC3_GCLK_ID].bit.CHEN == false) { ; }
		

	// Set PWM to 2.9kHz in 16Bit mode
	TC3->COUNT16.CTRLA.reg = TC_CTRLA_MODE_COUNT16 | TC_CTRLA_PRESCALER_DIV1;

	TC3->COUNT16.CC[1].reg = LED_IDLE_BRIGHTNESS;
	while(TC3->COUNT16.SYNCBUSY.bit.CC1) { ; }
		
	TC3->COUNT16.WAVE.bit.WAVEGEN = TC_WAVE_WAVEGEN_NPWM_Val;
		
	TC3->COUNT16.CTRLA.bit.ENABLE = true;
	while(TC3->COUNT16.SYNCBUSY.bit.ENABLE) { ; }
		
		
	PORTA->PMUX[LED_D5_5V_PA15_Pos/2].bit.PMUXO	= PMUX_E;
	PORTA->PINCFG[LED_D5_5V_PA15_Pos].reg		= PORT_PINCFG_PMUXEN;

}


// ----------------------------------------------------------------------------------------------


static int			LedFlashCounter;
static Timer_Type	LedErrorOffDelayTimer;
static _Bool		LedErrorIsAktiv;
static _Bool		LedErrorIsAktiv;


void StatusLedMain(void)
{
	static int			SmStatusLed;
	static Timer_Type	LedFlashTimer;
	
	
	
	if (TC3->COUNT16.STATUS.bit.CCBUFV1)
		return;
		
	
	switch (SmStatusLed)
	{
		case 0:
			if (LedErrorIsAktiv)
				if (TimerTest(&LedErrorOffDelayTimer))
					LedErrorIsAktiv = false;
			
			if ((LedFlashCounter == 0) && (!LedErrorIsAktiv))
			{
				if (TC3->COUNT16.CCBUF[1].reg != LED_IDLE_BRIGHTNESS)
					TC3->COUNT16.CCBUF[1].reg = LED_IDLE_BRIGHTNESS;
				
				break;
			}
			
			if (LedFlashCounter > 0)
			{
				LedFlashCounter	+= -1;
				
				TC3->COUNT16.CCBUF[1].reg = LED_FLASH_BRIGHTNESS;
			}
			else
			{
				TC3->COUNT16.CCBUF[1].reg = LED_IDLE_BRIGHTNESS;;
			}
				
			TimerStart_ms(LED_FLASH_ON_TIME, &LedFlashTimer);

			SmStatusLed = 1;
		break;
		

		case 1:	// Flash off
			if (!TimerTest(&LedFlashTimer))
				break;
		
			if (LedErrorIsAktiv)
				TC3->COUNT16.CCBUF[1].reg = 0;
			else
				TC3->COUNT16.CCBUF[1].reg = LED_IDLE_BRIGHTNESS;
		
			TimerStart_ms(LED_FLASH_OFF_TIME, &LedFlashTimer);
		
			SmStatusLed = 2;
		break;


		case 2:	// Off time
			if (!TimerTest(&LedFlashTimer))
				break;
			
			SmStatusLed = 0;	
		break;
	}
}



void StatusLedAddOneFlash(void)
{
	if (LedFlashCounter < LED_FLASH_COUNTER_MAX)
		LedFlashCounter += 1;
}


void StatusLedSignalError(void)
{
	TimerStart_ms(LED_ERROR_FLASH_TIME, &LedErrorOffDelayTimer);
	LedErrorIsAktiv = true;	
}
