/*
 * Time.c
 *
 * Created: 18.11.2018 11:24:54
 *  Author: Thomas
 */ 

#include <stdbool.h>
#include "sam.h"

#include "IO.h"
#include "clocks.h"
#include "misc.h"
#include "Time.h"



void TimeInit(void)
{
	// Enable module clock for TCs
	MCLK->APBAMASK.bit.TC0_ = true;
	MCLK->APBAMASK.bit.TC1_ = true;
	
	// Turn on clock for TCs core
	GCLK->PCHCTRL[TC0_GCLK_ID].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_120M)
								   | GCLK_PCHCTRL_CHEN;
	
	while(GCLK->PCHCTRL[TC0_GCLK_ID].bit.CHEN == false) { ; }
			
	// Reset	
	TC0->COUNT32.CTRLA.bit.SWRST = true;
	TC1->COUNT32.CTRLA.bit.SWRST = true;
	while(TC0->COUNT32.SYNCBUSY.bit.SWRST) { ; }
	while(TC1->COUNT32.SYNCBUSY.bit.SWRST) { ; }
			
	// Set timer 32Bit mode
	TC0->COUNT32.CTRLA.reg = TC_CTRLA_MODE_COUNT32 | TC_CTRLA_PRESCALER_DIV1;
	
	// Start counter
	TC0->COUNT32.CTRLA.bit.ENABLE = true;
	while(TC0->COUNT32.SYNCBUSY.bit.ENABLE) { ; }
}



#pragma GCC push_options
#pragma GCC optimize ("-Os")
void RAM_FUNC Delay_us(uint32_t delay)
{
	ATOMIC_BLOCK()
	{
		TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
		while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
	
		TC0->COUNT32.CC[0].reg = TC0->COUNT32.COUNT.reg + ((TIME_BASE / 1000000) * delay) - 44;

		TC0->COUNT32.INTFLAG.reg = TC_INTFLAG_MC0;
	}

	while((TC0->COUNT32.INTFLAG.reg & TC_INTFLAG_MC0) == 0) { ; }
}
#pragma GCC pop_options


void Delay_ms(uint32_t delay)
{
	Delay_us(delay * 1000);
}



uint32_t Stopwatch(uint32_t* lastCount)
{
	TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
	while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
		
	uint32_t count = TC0->COUNT32.COUNT.reg;
	
	uint32_t elapsedTicks = count - *lastCount;
	
	*lastCount = count;
	
	return elapsedTicks;
}

uint32_t Stopwatch_us(uint32_t* lastCount)
{
	return (Stopwatch(lastCount) / (TIME_BASE / 1000000));
}

uint32_t Stopwatch_ms(uint32_t* lastCount)
{
	return (Stopwatch(lastCount) / (TIME_BASE / 1000));
}


uint32_t IntervalTimer_ms(uint32_t* lastCount, uint32_t ms)
{
	TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
	while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
		
	uint32_t count = TC0->COUNT32.COUNT.reg;

	if (ms == 0)
	{
		*lastCount = count;
		return 0;
	}

	uint32_t elapsedTicks = count - *lastCount;

	uint32_t intervalTime = ms * (TIME_BASE / 1000);

	uint32_t intervals = 0;

	if (elapsedTicks >= intervalTime)
	{
		do
		{
			intervals++;
			*lastCount += intervalTime;
			elapsedTicks -= intervalTime;
		
		} while (elapsedTicks >= intervalTime);
	}
		
	return intervals;
}


void TimerStart_us(uint32_t time_us, Timer_Type* timer)
{
	TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
	while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
	
	timer->LastCount = TC0->COUNT32.COUNT.reg;
	
	timer->TimeTicks = time_us * (TIME_BASE / 1000000);
}

void TimerStart_ms(uint32_t time_ms, Timer_Type* timer)
{
	TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
	while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
		
	timer->LastCount = TC0->COUNT32.COUNT.reg;
	
	timer->TimeTicks = time_ms * (TIME_BASE / 1000);
}

_Bool TimerTest(Timer_Type* timer)
{
	TC0->COUNT32.CTRLBSET.bit.CMD = TC_CTRLBSET_CMD_READSYNC_Val;
	while(TC0->COUNT32.CTRLBSET.bit.CMD) { ; }
		
	if((TC0->COUNT32.COUNT.reg - timer->LastCount) >= timer->TimeTicks)
	{
		return true;	
	}
	
	return false;
}


