/*
 * W5500.c
 *
 * Created: 05.11.2018 18:08:33
 *  Author: Thomas
 */ 

#include <stdbool.h>
#include "sam.h"

#include "clocks.h"
#include "misc.h"
#include "IO.h"
#include "DMA/dma.h"
#include "Time/Time.h"
#include "W5500.h"


extern DmacDescriptors_t WsDmacDescriptors;


#define CHECK_RETV_RETV(x)	do { int ret_value_func = (x); if (ret_value_func < 0) { return (ret_value_func); } } while(0);


static _Bool InitOk;

static _Bool ReadWriteIsActive;

static struct 
{
	WNET_RW_ASYNC_RESULT_Type AsyncResult;
	int			RwWordSize;
	int			RwByteSize;
	uint8_t*	RwData;
	_Bool		RwRead;
	WnetRwAsyncCallBack_Type RwCallBack;
} WnAsyncInfo;

static uint32_t DummyRead;
static uint32_t DummyWrite;
static uint32_t SpiFrameStart;


// ----------------------------------------------------------------------------------------------


static void WNetDmaInit(void)
{
	// Channel 3 SPI read
	DMAC->Channel[3].CHCTRLA.reg  = DMAC_CHCTRLA_TRIGSRC(SERCOM1_DMAC_ID_RX)
								  | DMAC_CHCTRLA_TRIGACT_BURST
								  | DMAC_CHCTRLA_BURSTLEN_SINGLE
								  | DMAC_CHCTRLA_THRESHOLD_1BEAT;
		
	DMAC->Channel[3].CHPRILVL.reg = DMAC_CHPRILVL_PRILVL_LVL2;
	DMAC->Channel[3].CHEVCTRL.reg = DMAC_CHEVCTRL_EVACT_NOACT;
	DMAC->Channel[3].CHINTENSET.reg = DMAC_CHINTENSET_TCMPL | DMAC_CHINTENSET_TERR;
		
	WsDmacDescriptors.C3D1.SRCADDR.reg	= (uint32_t)&SERCOM1->SPI.DATA;
	WsDmacDescriptors.C3D1.DESCADDR.reg = 0;

	__NVIC_SetPriority(DMAC_3_IRQn, INT_PRIO_BG_M);
	__NVIC_EnableIRQ(DMAC_3_IRQn);


	// Channel 4 SPI write
	DMAC->Channel[4].CHCTRLA.reg  = DMAC_CHCTRLA_TRIGSRC(SERCOM1_DMAC_ID_TX)
								  | DMAC_CHCTRLA_TRIGACT_BURST
								  | DMAC_CHCTRLA_BURSTLEN_SINGLE
								  | DMAC_CHCTRLA_THRESHOLD_1BEAT;
	
	DMAC->Channel[4].CHPRILVL.reg = DMAC_CHPRILVL_PRILVL_LVL2;
	DMAC->Channel[4].CHEVCTRL.reg = DMAC_CHEVCTRL_EVACT_NOACT;
	
	WsDmacDescriptors.C4D1.DSTADDR.reg = (uint32_t)&SERCOM1->SPI.DATA;
	WsDmacDescriptors.C4D1.DESCADDR.reg = 0;
}



static void WNetSpiInit(void)
{

	// Port Settings
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	PORTA->DIRSET.reg = WN_CS_A0_PA02;
	PORTA->PINCFG[WN_CS_A0_PA02_Pos].reg = PORT_PINCFG_DRVSTR;
		
	PORTA->PMUX[WN_MOSI_PA00_Pos/2].bit.PMUXE	= PMUX_D;
	PORTA->PMUX[WN_SCK_PA01_Pos/2].bit.PMUXO	= PMUX_D;
	PORTB->PMUX[WN_MISO_PB23_Pos/2].bit.PMUXO	= PMUX_C;
	
	PORTA->PINCFG[WN_MOSI_PA00_Pos].reg = PORT_PINCFG_PMUXEN | PORT_PINCFG_DRVSTR;
	PORTA->PINCFG[WN_SCK_PA01_Pos].reg  = PORT_PINCFG_PMUXEN | PORT_PINCFG_DRVSTR;
	PORTB->PINCFG[WN_MISO_PB23_Pos].reg = PORT_PINCFG_PMUXEN;
	


	// Enable module clock
	MCLK->APBAMASK.bit.SERCOM1_ = true;

	// Turn on core clock
	GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN(GEN_GCLK_192M)
											| GCLK_PCHCTRL_CHEN;

	while(GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].bit.CHEN == false) { ; }


	// SPI Master mode 0, MSB first
	SERCOM1->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_DOPO(0)		// PAD0-MOSI / PAD1-SCK
						   | SERCOM_SPI_CTRLA_DIPO(3)		// PAD3-MISO
						   | SERCOM_SPI_CTRLA_MODE(3);		// SPI Master mode

	SERCOM1->SPI.CTRLC.reg = SERCOM_SPI_CTRLC_DATA32B		// Four bytes at once
						   | SERCOM_SPI_CTRLC_ICSPACE(1);	// Workaround that LENGTH is working
							
	SERCOM1->SPI.CTRLB.reg = SERCOM_SPI_CTRLB_CHSIZE(0)	// 8-Bit
						   | SERCOM_SPI_CTRLB_RXEN;
						   
	while(SERCOM1->SPI.SYNCBUSY.bit.CTRLB) { ; };

	// Baud register value corresponds to the SPI speed
	//SERCOM1->SPI.BAUD.reg = (192000000 / (2*8000000)) - 1;	// 8M Baud
	//SERCOM1->SPI.BAUD.reg = (192000000 / (2*24000000)) - 1;	// 24M Baud
	//SERCOM1->SPI.BAUD.reg = (120000000 / (2*30000000)) - 1;	// 30M Baud
	SERCOM1->SPI.BAUD.reg = (192000000 / (2*32000000)) - 1;	// 32M Baud

	SERCOM1->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
	while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };
}



int WNetInit(void)
{
	WNetDmaInit();
	WNetSpiInit();

	// W5500 reset
	PORTB->OUTCLR.reg = WN_RST_D3_PB22;
	PORTB->DIRSET.reg = WN_RST_D3_PB22;
	Delay_ms(1);
	PORTB->DIRCLR.reg = WN_RST_D3_PB22;
	Delay_ms(10);
	
	InitOk = true;
	
	return WNET_NOERR;
}



int WNetSocketInit(WNET_SOCKET_INIT_Type* socketInit)
{
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S0REG, socketInit->Ch0RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S0REG, socketInit->Ch0TxBufferSize));
	
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S1REG, socketInit->Ch1RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S1REG, socketInit->Ch1TxBufferSize));
	
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S2REG, socketInit->Ch2RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S2REG, socketInit->Ch2TxBufferSize));
	
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S3REG, socketInit->Ch3RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S3REG, socketInit->Ch3TxBufferSize));

	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S4REG, socketInit->Ch4RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S4REG, socketInit->Ch4TxBufferSize));

	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S5REG, socketInit->Ch5RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S5REG, socketInit->Ch5TxBufferSize));

	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S6REG, socketInit->Ch6RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S6REG, socketInit->Ch6TxBufferSize));

	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_RXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S7REG, socketInit->Ch7RxBufferSize));
	CHECK_RETV_RETV(WNetWriteByte(WNET_SRB_TXBUF_SIZE_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_S7REG, socketInit->Ch7TxBufferSize));
	
	return WNET_NOERR;
}



// ----------------------------------------------------------------------------------------------


WNET_RW_ASYNC_RESULT_Type WNetGetAsyncResult(_Bool peek)
{
	WNET_RW_ASYNC_RESULT_Type asyncResult = WnAsyncInfo.AsyncResult;
	
	if (!peek && !WnAsyncInfo.AsyncResult.IsBusy)
	{
		WnAsyncInfo.AsyncResult.IsCompleted	= false;
		WnAsyncInfo.AsyncResult.AsyncState	= 0;
	}
	
	return asyncResult;
}


void WNetEnterCriticalSectionAsyncResult(uint32_t* cs)
{
	ATOMIC_BLOCK()
	{
		*cs = __NVIC_GetEnableIRQ(DMAC_3_IRQn);
	
		if (*cs) { __NVIC_DisableIRQ(DMAC_3_IRQn); }
	}
}


void WNetLeaveCriticalSectionAsyncResult(uint32_t* cs)
{
	if (*cs) { 	ATOMIC_BLOCK() { __NVIC_EnableIRQ(DMAC_3_IRQn); } }
}


// ----------------------------------------------------------------------------------------------


int WNetBeginWriteBytes(uint16_t wNetAddress, uint8_t controlPhaseBsel, void* data, uint16_t size, WnetRwAsyncCallBack_Type callBack)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (data == 0)
		return WNET_ERROR_PARAMETER3;
	else if (size < 1 || size > 0x4000)	// > 16k
		return WNET_ERROR_PARAMETER4;

	ReadWriteIsActive = true;

	WnAsyncInfo.AsyncResult.IsCompleted	= false;
	WnAsyncInfo.AsyncResult.IsBusy		= true;
	WnAsyncInfo.AsyncResult.AsyncState	= WNET_NOERR;

	WnAsyncInfo.RwRead			= false;
	WnAsyncInfo.RwWordSize		= size >> 2;
	WnAsyncInfo.RwByteSize		= size & 0x3;
	WnAsyncInfo.RwData			= (uint8_t*)data;
	WnAsyncInfo.RwCallBack		= callBack;

	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	SpiFrameStart   = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_WRITE ) << 16);

	WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
										| DMAC_BTCTRL_BLOCKACT_INT
										| DMAC_BTCTRL_VALID;
	WsDmacDescriptors.C3D1.BTCNT.reg   = 1;
	WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)&DummyRead;
	
	WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
										| DMAC_BTCTRL_VALID;
	WsDmacDescriptors.C4D1.BTCNT.reg   = 1;
	WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)&SpiFrameStart;

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(3) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
	DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;

	return WNET_NOERR;
}

static void DmaWrite(void)
{
	if (WnAsyncInfo.RwWordSize)
	{
		WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
											| DMAC_BTCTRL_BLOCKACT_INT
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C3D1.BTCNT.reg   = WnAsyncInfo.RwWordSize;
		WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)&DummyRead;

		WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_SRCINC
											| DMAC_BTCTRL_BEATSIZE_WORD
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C4D1.BTCNT.reg   = WnAsyncInfo.RwWordSize;
		WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)WnAsyncInfo.RwData + (WnAsyncInfo.RwWordSize << 2);
		
		WnAsyncInfo.RwData += WnAsyncInfo.RwWordSize << 2;
		WnAsyncInfo.RwWordSize = 0;


		// Switch off "Inter-Character Spacing" it's 10% faster (works only with words)
		SERCOM1->SPI.CTRLA.bit.ENABLE = false;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };

		SERCOM1->SPI.CTRLC.reg = SERCOM_SPI_CTRLC_DATA32B;		// Four bytes at once

		SERCOM1->SPI.CTRLA.bit.ENABLE = true;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };

		SERCOM1->SPI.LENGTH.reg = 0;	// SERCOM_SPI_LENGTH_LENEN off
		while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }


		OPTIMIZATION_BARRIER
		DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
		DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;
		
		return;
	}
	
	if (WnAsyncInfo.RwByteSize)
	{
		WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_BYTE
											| DMAC_BTCTRL_BLOCKACT_INT
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C3D1.BTCNT.reg   = WnAsyncInfo.RwByteSize;
		WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)&DummyRead;

		WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_SRCINC
											| DMAC_BTCTRL_BEATSIZE_BYTE
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C4D1.BTCNT.reg   = WnAsyncInfo.RwByteSize;
		WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)WnAsyncInfo.RwData + WnAsyncInfo.RwByteSize;
		
		WnAsyncInfo.RwByteSize = 0;

		SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(1) | SERCOM_SPI_LENGTH_LENEN;
		while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

		OPTIMIZATION_BARRIER
		DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
		DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;

		return;
	}

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	ReadWriteIsActive = false;

	WnAsyncInfo.AsyncResult.AsyncState	= WNET_NOERR;
	WnAsyncInfo.AsyncResult.IsBusy		= false;
	WnAsyncInfo.AsyncResult.IsCompleted	= true;

	if(WnAsyncInfo.RwCallBack)	WnAsyncInfo.RwCallBack();
}



int WNetBeginReadBytes(uint16_t wNetAddress, uint8_t controlPhaseBsel, void* data, uint16_t size, WnetRwAsyncCallBack_Type callBack)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (data == 0)
		return WNET_ERROR_PARAMETER3;
	else if (size < 1 || size > 0x4000)	// > 16k
		return WNET_ERROR_PARAMETER4;	
	
	ReadWriteIsActive = true;
	
	WnAsyncInfo.AsyncResult.IsCompleted	= false;
	WnAsyncInfo.AsyncResult.IsBusy		= true;
	WnAsyncInfo.AsyncResult.AsyncState	= WNET_NOERR;

	WnAsyncInfo.RwRead			= true;
	WnAsyncInfo.RwWordSize		= size >> 2;
	WnAsyncInfo.RwByteSize		= size & 0x3;
	WnAsyncInfo.RwData			= (uint8_t*)data;
	WnAsyncInfo.RwCallBack		= callBack;

	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	SpiFrameStart   = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_READ ) << 16);

	WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
										| DMAC_BTCTRL_BLOCKACT_INT
										| DMAC_BTCTRL_VALID;
	WsDmacDescriptors.C3D1.BTCNT.reg   = 1;
	WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)&DummyRead;
	
	WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
										| DMAC_BTCTRL_VALID;
	WsDmacDescriptors.C4D1.BTCNT.reg   = 1;
	WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)&SpiFrameStart;

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(3) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
	DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;

	return WNET_NOERR;
}

static void DmaRead(void)
{
	if (WnAsyncInfo.RwWordSize)
	{
		WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_DSTINC
											| DMAC_BTCTRL_BEATSIZE_WORD
											| DMAC_BTCTRL_BLOCKACT_INT
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C3D1.BTCNT.reg   = WnAsyncInfo.RwWordSize;
		WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)WnAsyncInfo.RwData + (WnAsyncInfo.RwWordSize << 2);

		WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_WORD
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C4D1.BTCNT.reg   = WnAsyncInfo.RwWordSize;
		WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)&DummyWrite;
		
		WnAsyncInfo.RwData += WnAsyncInfo.RwWordSize << 2;
		WnAsyncInfo.RwWordSize = 0;


		// Switch off "Inter-Character Spacing" it's 10% faster (works only with words)
		SERCOM1->SPI.CTRLA.bit.ENABLE = false;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };

		SERCOM1->SPI.CTRLC.reg = SERCOM_SPI_CTRLC_DATA32B;		// Four bytes at once

		SERCOM1->SPI.CTRLA.bit.ENABLE = true;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };

		SERCOM1->SPI.LENGTH.reg = 0;	// SERCOM_SPI_LENGTH_LENEN off
		while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }
	

		OPTIMIZATION_BARRIER
		DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
		DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;
		
		return;
	}
	
	
	if (WnAsyncInfo.RwByteSize)
	{
		WsDmacDescriptors.C3D1.BTCTRL.reg =   DMAC_BTCTRL_DSTINC
											| DMAC_BTCTRL_BEATSIZE_BYTE
											| DMAC_BTCTRL_BLOCKACT_INT
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C3D1.BTCNT.reg   = WnAsyncInfo.RwByteSize;
		WsDmacDescriptors.C3D1.DSTADDR.reg = (uint32_t)WnAsyncInfo.RwData + WnAsyncInfo.RwByteSize;

		WsDmacDescriptors.C4D1.BTCTRL.reg =   DMAC_BTCTRL_BEATSIZE_BYTE
											| DMAC_BTCTRL_VALID;
		WsDmacDescriptors.C4D1.BTCNT.reg   = WnAsyncInfo.RwByteSize;
		WsDmacDescriptors.C4D1.SRCADDR.reg = (uint32_t)&DummyWrite;
		
		WnAsyncInfo.RwByteSize = 0;

		SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(1) | SERCOM_SPI_LENGTH_LENEN;
		while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

		OPTIMIZATION_BARRIER
		DMAC->Channel[3].CHCTRLA.bit.ENABLE = true;
		DMAC->Channel[4].CHCTRLA.bit.ENABLE = true;

		return;
	}

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	ReadWriteIsActive = false;

	WnAsyncInfo.AsyncResult.AsyncState	= WNET_NOERR;
	WnAsyncInfo.AsyncResult.IsBusy		= false;
	WnAsyncInfo.AsyncResult.IsCompleted	= true;

	if(WnAsyncInfo.RwCallBack)	WnAsyncInfo.RwCallBack();
}


// DMA read/write
ISR(DMAC_3_Handler)
{
	uint8_t CHINTFLAG = DMAC->Channel[3].CHINTFLAG.reg;
	DMAC->Channel[3].CHINTFLAG.reg = CHINTFLAG;

	// Switch on "Inter-Character Spacing" it was previews switch off (DmaRead/DmaWrite)
	if(SERCOM1->SPI.CTRLC.bit.ICSPACE == 0)
	{
		SERCOM1->SPI.CTRLA.bit.ENABLE = false;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };

		SERCOM1->SPI.CTRLC.reg  = SERCOM_SPI_CTRLC_DATA32B		// Four bytes at once
								| SERCOM_SPI_CTRLC_ICSPACE(1);	// Workaround that LENGTH is working

		SERCOM1->SPI.CTRLA.bit.ENABLE = true;
		while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) { ; };	
	}

	// If there was an error interrupt transmission
	if (CHINTFLAG & DMAC_CHINTFLAG_TERR)
	{
		OPTIMIZATION_BARRIER
		PORTA->OUTSET.reg = WN_CS_A0_PA02;
		OPTIMIZATION_BARRIER

		ReadWriteIsActive = false;

		WnAsyncInfo.AsyncResult.AsyncState	= WNET_ERROR_ASYNC_TERR;
		WnAsyncInfo.AsyncResult.IsBusy		= false;
		WnAsyncInfo.AsyncResult.IsCompleted	= true;

		if(WnAsyncInfo.RwCallBack)	WnAsyncInfo.RwCallBack();
	}
	else
	{
		if (WnAsyncInfo.RwRead)
		{
			DmaRead();
		}
		else
		{
			DmaWrite();
		}
	}
}


// ----------------------------------------------------------------------------------------------


int WNetWriteByte(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint8_t value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	
	ReadWriteIsActive = true;

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_WRITE ) << 16)
					| (value << 24);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(4) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.TXC == false) { ; }

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetWriteHWord(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint16_t value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	
	ReadWriteIsActive = true;

	value = __REV16(value);
	
	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_WRITE ) << 16)
					| (value << 24);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(5) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }

	SERCOM1->SPI.DATA.reg = value >> 8;
	while(SERCOM1->SPI.INTFLAG.bit.TXC == false) { ; }

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetWriteWord(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint32_t value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	
	ReadWriteIsActive = true;

	value = __REV(value);

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_WRITE ) << 16)
					| (value << 24);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(7) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }

	SERCOM1->SPI.DATA.reg = value >> 8;
	while(SERCOM1->SPI.INTFLAG.bit.TXC == false) { ; }

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER

	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetWriteBytes(uint16_t wNetAddress, uint8_t controlPhaseBsel, void* data, uint16_t size)
{
	uint8_t* bData = data;

	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (bData == 0)
		return WNET_ERROR_PARAMETER3;
	else if (size < 1 || size > 0x4000)	// > 16k
		return WNET_ERROR_PARAMETER4;

	ReadWriteIsActive = true;

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_WRITE ) << 16)
						| (*bData++ << 24);
	size -= 1;

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER


	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(4) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }


	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }

	int w = size & 0xfffffffc;

	if (w)
	{
		int i = 0;
		
		for (; i < w; i +=4)
		{
			uint32_t tmpData = (bData[i+0] <<  0)
							 | (bData[i+1] <<  8)
							 | (bData[i+2] << 16)
							 | (bData[i+3] << 24);
			
			SERCOM1->SPI.DATA.reg = tmpData;
			while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }
		}

		size -= i;
		bData += i;
	}

	while(SERCOM1->SPI.INTFLAG.bit.TXC == false) { ; }


	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(1) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }
	
	for (; size > 0; size--)
	{
		SERCOM1->SPI.DATA.reg = *bData++;
		while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }
	}

	while(SERCOM1->SPI.INTFLAG.bit.TXC == false) { ; }

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	ReadWriteIsActive = false;

	return WNET_NOERR;
}


// ----------------------------------------------------------------------------------------------


int WNetReadByte(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint8_t* value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (value == 0)
		return WNET_ERROR_PARAMETER3;
		
	ReadWriteIsActive = true;
	
	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_READ ) << 16);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(4) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }

	*value = (uint8_t)(SERCOM1->SPI.DATA.reg >> 24);

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetReadHWord(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint16_t* value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (value == 0)
		return WNET_ERROR_PARAMETER3;

	ReadWriteIsActive = true;

	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_READ ) << 16);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(5) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }
	SERCOM1->SPI.DATA.reg = 0;

	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
	uint16_t tmpValue = (uint8_t)(SERCOM1->SPI.DATA.reg >> 24);
	
	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
	tmpValue |= (uint8_t)SERCOM1->SPI.DATA.reg << 8;
	
	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	*value = __REV16(tmpValue);

	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetReadWord(uint16_t wNetAddress, uint8_t controlPhaseBsel, uint32_t* value)
{
	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (value == 0)
		return WNET_ERROR_PARAMETER3;

	ReadWriteIsActive = true;

	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_READ ) << 16);

	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(7) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }

	SERCOM1->SPI.DATA.reg = frData;
	while(SERCOM1->SPI.INTFLAG.bit.DRE == false) { ; }
	SERCOM1->SPI.DATA.reg = 0;

	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
	uint32_t tmpValue = (SERCOM1->SPI.DATA.reg >> 24);
	
	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
	tmpValue |= SERCOM1->SPI.DATA.reg << 8;
	
	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	*value = __REV(tmpValue);

	ReadWriteIsActive = false;
	
	return WNET_NOERR;
}


int WNetReadBytes(uint16_t wNetAddress, uint8_t controlPhaseBsel, void* data, uint16_t size)
{
	uint8_t* bData = data;

	if (!InitOk)
		return WNET_ERROR_NO_INIT;
	else if (ReadWriteIsActive)
		return WNET_ERROR_RW_ACTIVE;
	else if (bData == 0)
		return WNET_ERROR_PARAMETER3;
	else if (size < 1 || size > 0x4000)	// > 16k
		return WNET_ERROR_PARAMETER4;

	ReadWriteIsActive = true;

	while (SERCOM1->SPI.INTFLAG.bit.RXC)
	{
		DummyRead = SERCOM1->SPI.DATA.reg;
	}

	uint32_t frData = __REV16(wNetAddress)
					| ((  controlPhaseBsel
						| WNET_FRAME_CONTROL_PHASE_OM_VDLM
						| WNET_FRAME_CONTROL_PHASE_RW_READ ) << 16);
	
	OPTIMIZATION_BARRIER
	PORTA->OUTCLR.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER


	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(4) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }


	SERCOM1->SPI.DATA.reg = frData;
	
	size -= 1;
	int w = size & 0xfffffffc;
	
	while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }

	*bData++ = (SERCOM1->SPI.DATA.reg >> 24);
	
	if (w)
	{
		int i = 0;
	
		for (; i < w; i +=4)
		{
			SERCOM1->SPI.DATA.reg = 0;
			while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
		
			uint32_t tmpData = SERCOM1->SPI.DATA.reg;
		
			bData[i+0] = (uint8_t)(tmpData >>  0);
			bData[i+1] = (uint8_t)(tmpData >>  8);
			bData[i+2] = (uint8_t)(tmpData >> 16);
			bData[i+3] = (uint8_t)(tmpData >> 24);
		}
		
		size -= i;
		bData += i;
	}


	SERCOM1->SPI.LENGTH.reg = SERCOM_SPI_LENGTH_LEN(1) | SERCOM_SPI_LENGTH_LENEN;
	while (SERCOM1->SPI.SYNCBUSY.bit.LENGTH) { ; }
	
	for (; size > 0; size--)
	{
		SERCOM1->SPI.DATA.reg = 0;
		while(SERCOM1->SPI.INTFLAG.bit.RXC == false) { ; }
		
		*bData++ = (uint8_t)SERCOM1->SPI.DATA.reg;
	}

	OPTIMIZATION_BARRIER
	PORTA->OUTSET.reg = WN_CS_A0_PA02;
	OPTIMIZATION_BARRIER
	
	ReadWriteIsActive = false;

	return WNET_NOERR;
}


// ----------------------------------------------------------------------------------------------


int WNetWriteIpType(uint16_t wNetAddress, uint8_t controlPhaseBsel, WNET_IP_Type* ip)
{
	return WNetWriteBytes(wNetAddress, controlPhaseBsel, ip, 4);
}

 

int WNetReadIpType(uint16_t wNetAddress, uint8_t controlPhaseBsel, WNET_IP_Type* ip)
{
	return WNetReadBytes(wNetAddress, controlPhaseBsel, ip, 4);
}



int WNetWriteMacType(uint16_t wNetAddress, uint8_t controlPhaseBsel, WNET_MAC_Type* mac)
{
	return WNetWriteBytes(wNetAddress, controlPhaseBsel, mac, 6);
}



int WNetReadMacType(uint16_t wNetAddress, uint8_t controlPhaseBsel, WNET_MAC_Type* mac)
{
	return WNetReadBytes(wNetAddress, controlPhaseBsel, mac, 6);
}



int WNetGetStatus(WNET_CRB_PHYCFGR_Type* status)
{
	return WNetReadByte(WNET_CRB_PHYCFGR_ADDRESS, WNET_FRAME_CONTROL_PHASE_BSEL_CREG, &status->by);
}


