/*
 * W5500CRB.h
 *
 * Created: 01.11.2018 17:15:40
 *  Author: Thomas
 */ 


#ifndef W5500CRB_H_
#define W5500CRB_H_


#include <stdbool.h>
#include "sam.h"


// ----------------------------------------------------------------------------------------------


// MAC type
typedef	union {
	uint8_t MAC[6];
	uint8_t BYTE[6];
} WNET_MAC_Type;


// IP type
typedef	union {
	uint8_t  IP[4];
	uint8_t  BYTE[4];
} WNET_IP_Type;


// ----------------------------------------------------------------------------------------------


// Frame "Address Phase"
typedef union {
	struct {
		uint8_t HIGH;
		uint8_t LOW;
	} adr;
	uint8_t	BYTE[2];
	uint16_t hw;
} WNET_ADDRESS_Type;



// Frame "Control Phase"
typedef union {
	struct {
		uint8_t OM  :2;		// Operation Mode
		uint8_t RW  :1;		// Read/Write
		uint8_t BSEL:5;		// Block selection
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_FRAME_CONTROL_PHASE_Type;


#define WNET_FRAME_CONTROL_PHASE_OM_Pos				0			// Position "Operation Mode"
#define WNET_FRAME_CONTROL_PHASE_OM_Msk				(_U_(0x3) << WNET_FRAME_CONTROL_PHASE_OM_Pos)

#define	WNET_FRAME_CONTROL_PHASE_OM_VDLM_Val		_U_(0x0)	// Variable Data Length Mode, N-Bytes Data Phase (1 ? N)
#define	WNET_FRAME_CONTROL_PHASE_OM_FDLM1_Val		_U_(0x1)	// Fixed Data Length Mode , 1 Byte Data Length (N = 1)
#define	WNET_FRAME_CONTROL_PHASE_OM_FDLM2_Val		_U_(0x2)	// Fixed Data Length Mode , 2 Byte Data Length (N = 2)
#define	WNET_FRAME_CONTROL_PHASE_OM_FDLM4_Val		_U_(0x3)	// Fixed Data Length Mode , 4 Byte Data Length (N = 4)

#define WNET_FRAME_CONTROL_PHASE_OM_VDLM			(WNET_FRAME_CONTROL_PHASE_OM_VDLM_Val << WNET_FRAME_CONTROL_PHASE_OM_Pos)
#define WNET_FRAME_CONTROL_PHASE_OM_FDLM1			(WNET_FRAME_CONTROL_PHASE_OM_FDLM1_Val << WNET_FRAME_CONTROL_PHASE_OM_Pos)
#define WNET_FRAME_CONTROL_PHASE_OM_FDLM2			(WNET_FRAME_CONTROL_PHASE_OM_FDLM2_Val << WNET_FRAME_CONTROL_PHASE_OM_Pos)
#define WNET_FRAME_CONTROL_PHASE_OM_FDLM4			(WNET_FRAME_CONTROL_PHASE_OM_FDLM4_Val << WNET_FRAME_CONTROL_PHASE_OM_Pos)


#define WNET_FRAME_CONTROL_PHASE_RW_Pos				2			// Position "Read/Write"
#define WNET_FRAME_CONTROL_PHASE_RW_Msk				(_U_(0x1) << WNET_FRAME_CONTROL_PHASE_RW_Pos)

#define	WNET_FRAME_CONTROL_PHASE_RW_READ_Val		_U_(0x0)	
#define	WNET_FRAME_CONTROL_PHASE_RW_WRITE_Val		_U_(0x1)	

#define WNET_FRAME_CONTROL_PHASE_RW_READ			(WNET_FRAME_CONTROL_PHASE_RW_READ_Val << WNET_FRAME_CONTROL_PHASE_RW_Pos)
#define WNET_FRAME_CONTROL_PHASE_RW_WRITE			(WNET_FRAME_CONTROL_PHASE_RW_WRITE_Val << WNET_FRAME_CONTROL_PHASE_RW_Pos)


#define WNET_FRAME_CONTROL_PHASE_BSEL_Pos			3			// Position "Block selection"
#define WNET_FRAME_CONTROL_PHASE_BSEL_Msk			(_U_(0x1f) << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)


#define	WNET_FRAME_CONTROL_PHASE_BSEL_SXREG(value)	(((value) * 4 + 1) << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)	// Selects Socket x Register
#define	WNET_FRAME_CONTROL_PHASE_BSEL_SXTXB(value)	(((value) * 4 + 2) << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)	// Selects Socket x TX Buffer
#define	WNET_FRAME_CONTROL_PHASE_BSEL_SXRXB(value)	(((value) * 4 + 3) << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)	// Selects Socket x RX Buffer

#define	WNET_FRAME_CONTROL_PHASE_BSEL_CREG_Val		_U_(0x00)	// Selects Common Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0REG_Val		_U_(0x01)	// Selects Socket 0 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0TXB_Val		_U_(0x02)	// Selects Socket 0 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0RXB_Val		_U_(0x03)	// Selects Socket 0 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1REG_Val		_U_(0x05)	// Selects Socket 1 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1TXB_Val		_U_(0x06)	// Selects Socket 1 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1RXB_Val		_U_(0x07)	// Selects Socket 1 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2REG_Val		_U_(0x09)	// Selects Socket 2 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2TXB_Val		_U_(0x0a)	// Selects Socket 2 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2RXB_Val		_U_(0x0b)	// Selects Socket 2 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3REG_Val		_U_(0x0d)	// Selects Socket 3 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3TXB_Val		_U_(0x0e)	// Selects Socket 3 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3RXB_Val		_U_(0x0f)	// Selects Socket 3 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4REG_Val		_U_(0x11)	// Selects Socket 4 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4TXB_Val		_U_(0x12)	// Selects Socket 4 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4RXB_Val		_U_(0x13)	// Selects Socket 4 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5REG_Val		_U_(0x15)	// Selects Socket 5 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5TXB_Val		_U_(0x16)	// Selects Socket 5 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5RXB_Val		_U_(0x17)	// Selects Socket 5 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6REG_Val		_U_(0x19)	// Selects Socket 6 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6TXB_Val		_U_(0x1a)	// Selects Socket 6 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6RXB_Val		_U_(0x1b)	// Selects Socket 6 RX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7REG_Val		_U_(0x1d)	// Selects Socket 7 Register
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7TXB_Val		_U_(0x1e)	// Selects Socket 7 TX Buffer
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7RXB_Val		_U_(0x1f)	// Selects Socket 7 RX Buffer

#define WNET_FRAME_CONTROL_PHASE_BSEL_CREG		(WNET_FRAME_CONTROL_PHASE_BSEL_CREG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S0REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S0TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S0RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S0RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S1REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S1TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S1RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S1RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S2REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S2TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S2RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S2RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S3REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S3TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S3RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S3RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S4REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S4TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S4RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S4RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S5REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S5TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S5RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S5RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S6REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S6TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S6RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S6RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7REG		(WNET_FRAME_CONTROL_PHASE_BSEL_S7REG_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7TXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S7TXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)
#define WNET_FRAME_CONTROL_PHASE_BSEL_S7RXB		(WNET_FRAME_CONTROL_PHASE_BSEL_S7RXB_Val << WNET_FRAME_CONTROL_PHASE_BSEL_Pos)


// ----------------------------------------------------------------------------------------------

// Common Register Block


// Mode Register
typedef union {
	struct {
		uint8_t       :1;	// Res.
		uint8_t FARP  :1;	// Force ARP
		uint8_t       :1;	// Res.
		uint8_t PPPOE :1;	// PPPoE Mode
		uint8_t PB    :1;	// Ping Block Mode
		uint8_t WOL   :1;	// Wake on LAN
		uint8_t       :1;	// Res.
		uint8_t RST   :1;	// All internal registers will be initialized
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_MR_Type;

#define WNET_CRB_MR_OFFSET			0x0000	// Offset "Mode Register"
#define WNET_CRB_MR_ADDRESS			0x0000


#define WNET_CRB_MR_FARP_Pos		1		// Position "Force ARP"
#define WNET_CRB_MR_FARP			(_U_(0x1) << WNET_CRB_MR_FARP_Pos)

#define WNET_CRB_MR_PPPOE_Pos		3		// Position "PPPoE Mode"
#define WNET_CRB_MR_PPPOE			(_U_(0x1) << WNET_CRB_MR_PPPOE_Pos)

#define WNET_CRB_MR_PB_Pos			4		// Position "Ping Block Mode"
#define WNET_CRB_MR_PB				(_U_(0x1) << WNET_CRB_MR_PB_Pos)

#define WNET_CRB_MR_WOL_Pos			5		// Position "Ping Block Mode"
#define WNET_CRB_MR_WOL				(_U_(0x1) << WNET_CRB_MR_WOL_Pos)

#define WNET_CRB_MR_RST_Pos			7		// Position "Reset"
#define WNET_CRB_MR_RST				(_U_(0x1) << WNET_CRB_MR_RST_Pos)


// ----------------------------------------------------------------------------------------------


// Gateway Address Register
typedef union {
	WNET_IP_Type GAR;	
} WNET_CRB_GAR_Type;

#define WNET_CRB_GAR_OFFSET			0x0001	// Offset "Gateway Address Register"
#define WNET_CRB_GAR_ADDRESS		0x0001


// Subnet Mask Address Register
typedef union {
	WNET_IP_Type SUBR;	
} WNET_CRB_SUBR_Type;

#define WNET_CRB_SUBR_OFFSET		0x0005	// Offset "Subnet Mask Address Register"
#define WNET_CRB_SUBR_ADDRESS		0x0005


// Source MAC Address Register
typedef union {
	WNET_MAC_Type SHAR;	
} WNET_CRB_SHAR_Type;

#define WNET_CRB_SHAR_OFFSET		0x0009	// Offset "Subnet Mask Address Register"
#define WNET_CRB_SHAR_ADDRESS		0x0009


// Source IP Address Register
typedef union {
	WNET_IP_Type SIPR;
} WNET_CRB_SIPR_Type;

#define WNET_CRB_SIPR_OFFSET		0x000f	// Offset "Source IP Address Register"
#define WNET_CRB_SIPR_ADDRESS		0x000f


// Interrupt Low Level Timer Register
typedef union {
	//WNET_HWORD_Type INTLEVELR;
	uint16_t INTLEVELR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_CRB_INTLEVELR_Type;

#define WNET_CRB_INTLEVELR_OFFSET	0x0013	// Offset "Interrupt Low Level Timer Register"
#define WNET_CRB_INTLEVELR_ADDRESS	0x0013


// Interrupt Register
typedef union {
	struct {
		uint8_t          :4;	// Res.
		uint8_t MP       :1;	// Magic Packet
		uint8_t PPPOE    :1;	// PPPoE Connection Close
		uint8_t UNREACH  :1;	// Destination unreachable
		uint8_t CONFLICT :1;	// IP Conflict
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_IR_Type;

#define WNET_CRB_IR_OFFSET			0x0015	// Offset "Interrupt Register"
#define WNET_CRB_IR_ADDRESS			0x0015


#define WNET_CRB_IR_MP_Pos			4		// Position "Magic Packet"
#define WNET_CRB_IR_MP				(_U_(0x1) << WNET_CRB_IR_MP_Pos)

#define WNET_CRB_IR_PPPOE_Pos		5		// Position "PPPoE Connection Close"
#define WNET_CRB_IR_PPPOE			(_U_(0x1) << WNET_CRB_IR_PPPOE_Pos)

#define WNET_CRB_IR_UNREACH_Pos		6		// Position "Destination unreachable"
#define WNET_CRB_IR_UNREACH			(_U_(0x1) << WNET_CRB_IR_UNREACH_Pos)

#define WNET_CRB_IR_CONFLICT_Pos	7		// Position "IP Conflict"
#define WNET_CRB_IR_CONFLICT		(_U_(0x1) << WNET_CRB_IR_CONFLICT_Pos)


// Interrupt Mask Register
typedef union {
	struct {
		uint8_t          :4;	// Res.
		uint8_t MP       :1;	// Magic Packet
		uint8_t PPPOE    :1;	// PPPoE Connection Close
		uint8_t UNREACH  :1;	// Destination unreachable
		uint8_t CONFLICT :1;	// IP Conflict
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_IMR_Type;

#define WNET_CRB_IMR_OFFSET			0x0016	// Offset "Interrupt Mask Register"
#define WNET_CRB_IMR_ADDRESS		0x0016


#define WNET_CRB_IMR_MP_Pos			4		// Position "Magic Packet"
#define WNET_CRB_IMR_MP				(_U_(0x1) << WNET_CRB_IMR_MP_Pos)

#define WNET_CRB_IMR_PPPOE_Pos		5		// Position "PPPoE Connection Close"
#define WNET_CRB_IMR_PPPOE			(_U_(0x1) << WNET_CRB_IMR_PPPOE_Pos)

#define WNET_CRB_IMR_UNREACH_Pos	6		// Position "Destination unreachable"
#define WNET_CRB_IMR_UNREACH		(_U_(0x1) << WNET_CRB_IMR_UNREACH_Pos)

#define WNET_CRB_IMR_CONFLICT_Pos	7		// Position "IP Conflict"
#define WNET_CRB_IMR_CONFLICT		(_U_(0x1) << WNET_CRB_IMR_CONFLICT_Pos)


// Socket Interrupt Register
typedef union {
	struct {
		uint8_t S0 :1;	// Socket 0 Interrupt
		uint8_t S1 :1;	// Socket 1 Interrupt
		uint8_t S2 :1;	// Socket 2 Interrupt
		uint8_t S3 :1;	// Socket 3 Interrupt
		uint8_t S4 :1;	// Socket 4 Interrupt
		uint8_t S5 :1;	// Socket 5 Interrupt
		uint8_t S6 :1;	// Socket 6 Interrupt
		uint8_t S7 :1;	// Socket 7 Interrupt
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_SIR_Type;

#define WNET_CRB_SIR_OFFSET			0x0017	// Offset "Socket Interrupt Register"
#define WNET_CRB_SIR_ADDRESS		0x0017


#define WNET_CRB_SIR_S0_Pos			0		// Position "Socket 0 Interrupt"
#define WNET_CRB_SIR_S0				(_U_(0x1) << WNET_CRB_SIR_S0_Pos)
#define WNET_CRB_SIR_S1_Pos			1		// Position "Socket 1 Interrupt"
#define WNET_CRB_SIR_S1				(_U_(0x1) << WNET_CRB_SIR_S1_Pos)
#define WNET_CRB_SIR_S2_Pos			2		// Position "Socket 1 Interrupt"
#define WNET_CRB_SIR_S2				(_U_(0x1) << WNET_CRB_SIR_S1_Pos)
#define WNET_CRB_SIR_S3_Pos			3		// Position "Socket 2 Interrupt"
#define WNET_CRB_SIR_S3				(_U_(0x1) << WNET_CRB_SIR_S2_Pos)
#define WNET_CRB_SIR_S4_Pos			4		// Position "Socket 3 Interrupt"
#define WNET_CRB_SIR_S4				(_U_(0x1) << WNET_CRB_SIR_S3_Pos)
#define WNET_CRB_SIR_S5_Pos			5		// Position "Socket 4 Interrupt"
#define WNET_CRB_SIR_S5				(_U_(0x1) << WNET_CRB_SIR_S4_Pos)
#define WNET_CRB_SIR_S6_Pos			6		// Position "Socket 5 Interrupt"
#define WNET_CRB_SIR_S6				(_U_(0x1) << WNET_CRB_SIR_S5_Pos)
#define WNET_CRB_SIR_S7_Pos			7		// Position "Socket 6 Interrupt"
#define WNET_CRB_SIR_S7				(_U_(0x1) << WNET_CRB_SIR_S5_Pos)


// Socket Interrupt Mask Register
typedef union {
	struct {
		uint8_t S0 :1;	// Socket 0 Interrupt Mask
		uint8_t S1 :1;	// Socket 1 Interrupt Mask
		uint8_t S2 :1;	// Socket 2 Interrupt Mask
		uint8_t S3 :1;	// Socket 3 Interrupt Mask
		uint8_t S4 :1;	// Socket 4 Interrupt Mask
		uint8_t S5 :1;	// Socket 5 Interrupt Mask
		uint8_t S6 :1;	// Socket 6 Interrupt Mask
		uint8_t S7 :1;	// Socket 7 Interrupt Mask
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_SIMR_Type;

#define WNET_CRB_SIMR_OFFSET		0x0018	// Offset "Socket Interrupt Mask Register"
#define WNET_CRB_SIMR_ADDRESS		0x0018


#define WNET_CRB_SIMR_S0_Pos		0		// Position "Socket 0 Mask Interrupt"
#define WNET_CRB_SIMR_S0			(_U_(0x1) << WNET_CRB_SIMR_S0_Pos)
#define WNET_CRB_SIMR_S1_Pos		1		// Position "Socket 1 Mask Interrupt"
#define WNET_CRB_SIMR_S1			(_U_(0x1) << WNET_CRB_SIMR_S1_Pos)
#define WNET_CRB_SIMR_S2_Pos		2		// Position "Socket 1 Mask Interrupt"
#define WNET_CRB_SIMR_S2			(_U_(0x1) << WNET_CRB_SIMR_S1_Pos)
#define WNET_CRB_SIMR_S3_Pos		3		// Position "Socket 2 Mask Interrupt"
#define WNET_CRB_SIMR_S3			(_U_(0x1) << WNET_CRB_SIMR_S2_Pos)
#define WNET_CRB_SIMR_S4_Pos		4		// Position "Socket 3 Mask Interrupt"
#define WNET_CRB_SIMR_S4			(_U_(0x1) << WNET_CRB_SIMR_S3_Pos)
#define WNET_CRB_SIMR_S5_Pos		5		// Position "Socket 4 Mask Interrupt"
#define WNET_CRB_SIMR_S5			(_U_(0x1) << WNET_CRB_SIMR_S4_Pos)
#define WNET_CRB_SIMR_S6_Pos		6		// Position "Socket 5 Mask Interrupt"
#define WNET_CRB_SIMR_S6			(_U_(0x1) << WNET_CRB_SIMR_S5_Pos)
#define WNET_CRB_SIMR_S7_Pos		7		// Position "Socket 6 Mask Interrupt"
#define WNET_CRB_SIMR_S7			(_U_(0x1) << WNET_CRB_SIMR_S5_Pos)


// Retry Time Register
typedef union {
	//WNET_HWORD_Type RTR;
	uint16_t RTR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_CRB_RTR_Type;

#define WNET_CRB_RTR_OFFSET			0x0019	// Offset "Retry Time Register"
#define WNET_CRB_RTR_ADDRESS		0x0019


// Retry Count Register
typedef union {
	//WNET_BYTE_Type RCR;
	uint8_t RCR;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_RCR_Type;

#define WNET_CRB_RCR_OFFSET			0x001b	// Offset "Retry Count Register"
#define WNET_CRB_RCR_ADDRESS		0x001b


// PPP Link Control Protocol Request Timer Register
typedef union {
	//WNET_BYTE_Type PTIMER;
	uint8_t PTIMER;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_PTIMER_Type;

#define WNET_CRB_PTIMER_OFFSET		0x001c	// Offset "PPP Link Control Protocol Request Timer Register"
#define WNET_CRB_PTIMER_ADDRESS		0x001c


// PPP Link Control Protocol Magic number Register
typedef union {
	//WNET_BYTE_Type PMAGIC;
	uint8_t PMAGIC;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_PMAGIC_Type;

#define WNET_CRB_PMAGIC_OFFSET		0x001d	// Offset "PPP Link Control Protocol Magic number Register"
#define WNET_CRB_PMAGIC_ADDRESS		0x001d


// PPP Destination MAC Address Register
typedef union {
	WNET_MAC_Type PHAR;
} WNET_CRB_PHAR_Type;

#define WNET_CRB_PHAR_OFFSET		0x001e	// Offset "PPP Destination MAC Address Register"
#define WNET_CRB_PHAR_ADDRESS		0x001e


// PPP Session Identification Register
typedef union {
	//WNET_HWORD_Type PSID;
	uint16_t PSID;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_CRB_PSID_Type;

#define WNET_CRB_PSID_OFFSET		0x0024	// Offset "PPP Session Identification"
#define WNET_CRB_PSID_ADDRESS		0x0024


// PPP Maximum Segment Size Register
typedef union {
	//WNET_HWORD_Type PMRU;
	uint16_t PMRU;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_CRB_PMRU_Type;

#define WNET_CRB_PMRU_OFFSET		0x0026	// Offset "PPP Maximum Segment Size Register"
#define WNET_CRB_PMRU_ADDRESS		0x0026


// Unreachable IP Address Register
typedef union {
	WNET_IP_Type UIPR;
} WNET_CRB_UIPR_Type;

#define WNET_CRB_UIPR_OFFSET		0x0028	// Offset "Unreachable IP Address Register"
#define WNET_CRB_UIPR_ADDRESS		0x0028


// Unreachable Port Register
typedef union {
	//WNET_HWORD_Type UPORTR;
	uint16_t UPORTR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_CRB_UPORTR_Type;

#define WNET_CRB_UPORTR_OFFSET		0x002c	// Offset "Unreachable Port Register"
#define WNET_CRB_UPORTR_ADDRESS		0x002c


// PHY Configuration Register
typedef union {
	struct {
		uint8_t LNK   :1;		// Link Status   [Read Only] (Link down/up)
		uint8_t SPD   :1;		// Speed Status  [Read Only] (10/100Mpbs based)
		uint8_t DPX   :1;		// Duplex Status [Read Only] (Half/Full duplex)
		uint8_t OPMDC :3;		// Operation Mode Configuration
		uint8_t OPMD  :1;		// Configure PHY Operation Mode
		uint8_t RST   :1;		// PHY reset
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_PHYCFGR_Type;

#define WNET_CRB_PHYCFGR_OFFSET		0x002e	// Offset "PHY Configuration Register"
#define WNET_CRB_PHYCFGR_ADDRESS	0x002e


#define WNET_CRB_PHYCFGR_LNK_Pos					0			// Position "Link Status"
#define WNET_CRB_PHYCFGR_LNK_Msk					(_U_(0x1) << WNET_CRB_PHYCFGR_LNK_Pos)

#define	WNET_CRB_PHYCFGR_LNK_DOWN_Val				_U_(0x0)
#define	WNET_CRB_PHYCFGR_LNK_UP_Val					_U_(0x1)

#define WNET_CRB_PHYCFGR_LNK_DOWN					(WNET_CRB_PHYCFGR_LNK_DOWN_Val << WNET_CRB_PHYCFGR_LNK_Pos)
#define WNET_CRB_PHYCFGR_LNK_UP						(WNET_CRB_PHYCFGR_LNK_UP_Val << WNET_CRB_PHYCFGR_LNK_Pos)


#define WNET_CRB_PHYCFGR_SPD_Pos					1			// Position "Speed Status"
#define WNET_CRB_PHYCFGR_SPD_Msk					(_U_(0x1) << WNET_CRB_PHYCFGR_SPD_Pos)

#define	WNET_CRB_PHYCFGR_SPD_10_Val					_U_(0x0)
#define	WNET_CRB_PHYCFGR_SPD_100_Val				_U_(0x1)

#define WNET_CRB_PHYCFGR_SPD_10						(WNET_CRB_PHYCFGR_SPD_10_Val << WNET_CRB_PHYCFGR_SPD_Pos)
#define WNET_CRB_PHYCFGR_SPD_100					(WNET_CRB_PHYCFGR_SPD_100_Val << WNET_CRB_PHYCFGR_SPD_Pos)


#define WNET_CRB_PHYCFGR_DPX_Pos					2			// Position "Duplex Status"
#define WNET_CRB_PHYCFGR_DPX_Msk					(_U_(0x1) << WNET_CRB_PHYCFGR_DPX_Pos)

#define	WNET_CRB_PHYCFGR_DPX_HALF_Val				_U_(0x0)
#define	WNET_CRB_PHYCFGR_DPX_FULL_Val				_U_(0x1)

#define WNET_CRB_PHYCFGR_DPX_HALF					(WNET_CRB_PHYCFGR_DPX_HALF_Val << WNET_CRB_PHYCFGR_DPX_Pos)
#define WNET_CRB_PHYCFGR_DPX_FULL					(WNET_CRB_PHYCFGR_DPX_FULL_Val << WNET_CRB_PHYCFGR_DPX_Pos)


#define WNET_CRB_PHYCFGR_OPMDC_Pos					3			// Position "Operation Mode Configuration"
#define WNET_CRB_PHYCFGR_OPMDC_Msk					(_U_(0x7) << WNET_CRB_PHYCFGR_OPMDC_Pos)

#define	WNET_CRB_PHYCFGR_OPMDC_10HDAND_Val			_U_(0x0)	// 10BT Half-duplex, Auto-negotiation disabled
#define	WNET_CRB_PHYCFGR_OPMDC_10FDAND_Val			_U_(0x1)	// 10BT Full-duplex, Auto-negotiation disabled
#define	WNET_CRB_PHYCFGR_OPMDC_100HDAND_Val			_U_(0x2)	// 100BT Half-duplex, Auto-negotiation disabled
#define	WNET_CRB_PHYCFGR_OPMDC_100FDAND_Val			_U_(0x3)	// 100BT Full-duplex, Auto-negotiation disabled
#define	WNET_CRB_PHYCFGR_OPMDC_100HDANE_Val			_U_(0x4)	// 100BT Half-duplex, Auto-negotiation enabled
#define	WNET_CRB_PHYCFGR_OPMDC_PWDOWN_Val			_U_(0x6)	// Power Down mode
#define	WNET_CRB_PHYCFGR_OPMDC_ACANE_Val			_U_(0x7)	// All capable, Auto-negotiation enabled

#define WNET_CRB_PHYCFGR_OPMDC_10HDAND				(WNET_CRB_PHYCFGR_OPMDC_10HDAND_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_10FDAND				(WNET_CRB_PHYCFGR_OPMDC_10FDAND_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_100HDAND				(WNET_CRB_PHYCFGR_OPMDC_100HDAND_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_100FDAND				(WNET_CRB_PHYCFGR_OPMDC_100FDAND_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_100HDANE				(WNET_CRB_PHYCFGR_OPMDC_100HDANE_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_PWDOWN				(WNET_CRB_PHYCFGR_OPMDC_PWDOWN_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)
#define	WNET_CRB_PHYCFGR_OPMDC_ACANE				(WNET_CRB_PHYCFGR_OPMDC_ACANE_Val << WNET_CRB_PHYCFGR_OPMDC_Pos)


#define WNET_CRB_PHYCFGR_OPMD_Pos					6			// Position "Configure PHY Operation Mode"
#define WNET_CRB_PHYCFGR_OPMD_Msk					(_U_(0x1) << WNET_CRB_PHYCFGR_OPMD_Pos)

#define	WNET_CRB_PHYCFGR_OPMD_HWPINS_Val			_U_(0x0)
#define	WNET_CRB_PHYCFGR_OPMD_OPMDC_Val				_U_(0x1)

#define WNET_CRB_PHYCFGR_OPMD_HWPINS				(WNET_CRB_PHYCFGR_OPMD_HWPINS_Val << WNET_CRB_PHYCFGR_OPMD_Pos)
#define WNET_CRB_PHYCFGR_OPMD_OPMDC					(WNET_CRB_PHYCFGR_OPMD_OPMDC_Val << WNET_CRB_PHYCFGR_OPMD_Pos)


#define WNET_CRB_PHYCFGR_RST_Pos					7			// Position "PHY reset"
#define WNET_CRB_PHYCFGR_RST						(_U_(0x1) << WNET_CRB_PHYCFGR_RST_Pos)


// Chip version register
typedef union {
	uint8_t VERSIONR;	// (W5500 = 0x04)
	uint8_t BYTE[1];
	uint8_t by;
} WNET_CRB_VERSIONR_Type;

#define WNET_CRB_VERSIONR_OFFSET		0x0039	// Offset "Chip version Register"
#define WNET_CRB_VERSIONR_ADDRESS		0x0039








#endif /* W5500CRB_H_ */