/*
 * W5500SRB.h
 *
 * Created: 01.11.2018 17:15:40
 *  Author: Thomas
 */ 


#ifndef W5500SRB_H_
#define W5500SRB_H_


#include <stdbool.h>
#include "sam.h"


// ----------------------------------------------------------------------------------------------

// Socket Register Block


// Mode Register
typedef union {
	union {	// MACRAW mode
		struct {
			uint8_t P      :4;	// Protocol mode of Socket
			uint8_t MIP6B  :1;	// IPv6 packet Blocking
			uint8_t MMB    :1;	// Multicast Blocking
			uint8_t BCASTB :1;	// Broadcast Blocking
			uint8_t MFEN   :1;	// MAC Filter Enable
		} b;
		uint8_t BYTE[1];
		uint8_t by;
	} MACRAW;
	union { // UDP mode
		struct {
			uint8_t P      :4;	// Protocol mode of Socket
			uint8_t UCASTB :1;	// UNICAST Blocking
			uint8_t MC     :1;	// Multicast version
			uint8_t BCASTB :1;	// Broadcast Blocking
			uint8_t MULTI  :1;	// Multicasting
		} b;
		uint8_t BYTE[1];
		uint8_t by;
	} UDP;
	union {	// TCP mode
		struct {
			uint8_t P      :4;	// Protocol mode of Socket
			uint8_t        :1;	// Res.
			uint8_t ND     :1;	// Use No Delayed ACK
			uint8_t        :1;	// Res.
			uint8_t        :1;	// Res.
		} b;
		uint8_t BYTE[1];
		uint8_t by;
	} TCP;
} WNET_SRB_MR_Type;

#define WNET_SRB_MR_OFFSET			0x0000	// Offset "Mode Register"
#define WNET_SRB_MR_ADDRESS			0x0000


#define WNET_SRB_MR_P_Pos					0			// Position "Protocol mode"
#define WNET_SRB_MR_P_Msk					(_U_(0x7) << WNET_SRB_MR_P_Pos)

#define	WNET_SRB_MR_P_CLOSE_Val				_U_(0x0)
#define	WNET_SRB_MR_P_TCP_Val				_U_(0x1)
#define	WNET_SRB_MR_P_UDP_Val				_U_(0x2)
#define	WNET_SRB_MR_P_IPRAW_Val				_U_(0x3)
#define	WNET_SRB_MR_P_MACRAW_Val			_U_(0x4)

#define WNET_SRB_MR_P_CLOSE					(WNET_SRB_MR_P_CLOSE_Val << WNET_SRB_MR_P_Pos)
#define WNET_SRB_MR_P_TCP					(WNET_SRB_MR_P_TCP_Val << WNET_SRB_MR_P_Pos)
#define	WNET_SRB_MR_P_UDP					(WNET_SRB_MR_P_UDP_Val << WNET_SRB_MR_P_Pos)
#define	WNET_SRB_MR_P_IPRAW					(WNET_SRB_MR_P_IPRAW_Val << WNET_SRB_MR_P_Pos)
#define	WNET_SRB_MR_P_MACRAW				(WNET_SRB_MR_P_MACRAW_Val << WNET_SRB_MR_P_Pos)


#define WNET_SRB_MR_MACRAW_MIP6B_Pos		4		// Position "IPv6 packet Blocking"
#define WNET_SRB_MR_MACRAW_MIP6B			(_U_(0x1) << WNET_SRB_MR_MACRAW_MIP6B_Pos)

#define WNET_SRB_MR_MACRAW_MMB_Pos			5		// Position "Multicast Blocking"
#define WNET_SRB_MR_MACRAW_MMB				(_U_(0x1) << WNET_SRB_MR_MACRAW_MMB_Pos)

#define WNET_SRB_MR_MACRAW_BCASTB_Pos		6		// Position "Broadcast Blocking"
#define WNET_SRB_MR_MACRAW_BCASTB			(_U_(0x1) << WNET_SRB_MR_MACRAW_BCASTB_Pos)

#define WNET_SRB_MR_MACRAW_MFEN_Pos			7		// Position "MAC Filter Enable"
#define WNET_SRB_MR_MACRAW_MFEN				(_U_(0x1) << WNET_SRB_MR_MACRAW_MFEN_Pos)


#define WNET_SRB_MR_UDP_UCASTB_Pos			4		// Position "UNICAST Blocking"
#define WNET_SRB_MR_UDP_UCASTB				(_U_(0x1) << WNET_SRB_MR_UDP_UCASTB_Pos)

#define WNET_SRB_MR_UDP_MC_Pos				5		// Position "Multicast version"
#define WNET_SRB_MR_UDP_MC_Msk				(_U_(0x1) << WNET_SRB_MR_P_Pos)

#define	WNET_SRB_MR_UDP_MC_IGMPV2_Val		_U_(0x0)
#define	WNET_SRB_MR_UDP_MC_IGMPV1_Val		_U_(0x1)

#define WNET_SRB_MR_UDP_MC_IGMPV2			(WNET_SRB_MR_UDP_MC_IGMPV2_Val << WNET_SRB_MR_UDP_MC_Pos)
#define WNET_SRB_MR_UDP_MC_IGMPV1			(WNET_SRB_MR_UDP_MC_IGMPV1_Val << WNET_SRB_MR_UDP_MC_Pos)

#define WNET_SRB_MR_UDP_BCASTB_Pos			6		// Position "Broadcast Blocking"
#define WNET_SRB_MR_UDP_BCASTB				(_U_(0x1) << WNET_SRB_MR_UDP_BCASTB_Pos)

#define WNET_SRB_MR_UDP_MULTI_Pos			7		// Position "Multicasting"
#define WNET_SRB_MR_UDP_MULTI				(_U_(0x1) << WNET_SRB_MR_UDP_MULTI_Pos)


#define WNET_SRB_MR_TCP_ND_Pos				5		// Position "Use No Delayed ACK"
#define WNET_SRB_MR_UDP_ND					(_U_(0x1) << WNET_SRB_MR_TCP_ND_Pos)


// Socket Command Register
typedef union {
	struct {
		uint8_t CMD    :8;	// Command
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_SCR_Type;

#define WNET_SRB_SCR_OFFSET					0x0001	// Offset "Socket Command Register"
#define WNET_SRB_SCR_ADDRESS				0x0001


#define WNET_SRB_SCR_CMD_Pos				0			// Position "Command"
#define WNET_SRB_SCR_CMD_Msk				(_U_(0xff) << WNET_SRB_SCR_CMD_Pos)

#define	WNET_SRB_SCR_CMD_OPEN_Val			_U_(0x01)	// Initialize or open socket
#define	WNET_SRB_SCR_CMD_LISTEN_Val			_U_(0x02)	// Wait connection request in TCP mode(Server mode)
#define	WNET_SRB_SCR_CMD_CONNECT_Val		_U_(0x04)	// Send connection request in TCP mode(Client mode)
#define	WNET_SRB_SCR_CMD_DISCON_Val			_U_(0x08)	// Send closing request in TCP mode
#define WNET_SRB_SCR_CMD_CLOSE_Val			_U_(0x10)	// Close socket
#define WNET_SRB_SCR_CMD_SEND_Val			_U_(0x20)	// Update Tx memory pointer and send data
#define WNET_SRB_SCR_CMD_SEND_MAC_Val		_U_(0x21)	// Send data with MAC address, so without ARP process
#define WNET_SRB_SCR_CMD_SEND_KEEP_Val		_U_(0x22)	// Send keep alive message
#define WNET_SRB_SCR_CMD_RECV_Val			_U_(0x40)	// Update Rx memory buffer pointer and receive data

#define WNET_SRB_SCR_CMD_OPEN				(WNET_SRB_SCR_CMD_OPEN_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_LISTEN				(WNET_SRB_SCR_CMD_LISTEN_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_CONNECT			(WNET_SRB_SCR_CMD_CONNECT_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_DISCON				(WNET_SRB_SCR_CMD_DISCON_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_CLOSE				(WNET_SRB_SCR_CMD_CLOSE_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_SEND				(WNET_SRB_SCR_CMD_SEND_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_SEND_MAC			(WNET_SRB_SCR_CMD_SEND_MAC_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_SEND_KEEP			(WNET_SRB_SCR_CMD_SEND_KEEP_Val << WNET_SRB_SCR_CMD_Pos)
#define WNET_SRB_SCR_CMD_RECV				(WNET_SRB_SCR_CMD_RECV_Val << WNET_SRB_SCR_CMD_Pos)


// Socket Interrupt Register
typedef union {
	struct {
		uint8_t CON     :1;		// Connection with peer
		uint8_t DISCON  :1;		// This is issued when FIN or FIN/ACK packet is received from a peer
		uint8_t RECV    :1;		// This is issued whenever data is received from a peer
		uint8_t TIMEOUT :1;		// This is issued when ARPTO or TCPTO occurs.
		uint8_t SEND_OK :1;		// This is issued when SEND command is completed.
		uint8_t	        :3;		// Res.
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_SIR_Type;

#define WNET_SRB_SIR_OFFSET				0x0002	// Offset "Socket Interrupt Register"
#define WNET_SRB_SIR_ADDRESS			0x0002


#define WNET_SRB_SIR_CON_Pos			0		// Position "Connection"
#define WNET_SRB_SIR_CON				(_U_(0x1) << WNET_SRB_SIR_CON_Pos)

#define WNET_SRB_SIR_DICON_Pos			1		// Position "Disconnection"
#define WNET_SRB_SIR_DICON				(_U_(0x1) << WNET_SRB_SIR_DICON_Pos)

#define WNET_SRB_SIR_RECV_Pos			2		// Position "Receive"
#define WNET_SRB_SIR_RECV				(_U_(0x1) << WNET_SRB_SIR_RECV_Pos)

#define WNET_SRB_SIR_TIMEOUT_Pos		3		// Position "Timeout"
#define WNET_SRB_SIR_TIMEOUT			(_U_(0x1) << WNET_SRB_SIR_TIMEOUT_Pos)

#define WNET_SRB_SIR_SEND_OK_Pos		4		// Position "Send OK"
#define WNET_SRB_SIR_SEND_OK			(_U_(0x1) << WNET_SRB_SIR_SEND_OK_Pos)


// Socket Status Register
typedef union {
	struct {
		uint8_t STAT :8;	// Status
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_SSR_Type;

#define WNET_SRB_SSR_OFFSET							0x0003	// Offset "Socket Status Register"
#define WNET_SRB_SSR_ADDRESS						0x0003


#define WNET_SRB_SSR_STAT_Pos						0			// Position "Socket Status Register"
#define WNET_SRB_SSR_STAT_Msk						(_U_(0xff) << WNET_SRB_SSR_STAT_Pos)

#define	WNET_SRB_SSR_STAT_SOCK_CLOSED_Val			_U_(0x00)
#define	WNET_SRB_SSR_STAT_SOCK_INIT_Val				_U_(0x13)
#define	WNET_SRB_SSR_STAT_SOCK_LISTEN_Val			_U_(0x14)
#define	WNET_SRB_SSR_STAT_SOCK_ESTABLISHED_Val		_U_(0x17)
#define	WNET_SRB_SSR_STAT_SOCK_CLOSE_WAIT_Val		_U_(0x1c)
#define	WNET_SRB_SSR_STAT_SOCK_UDP_Val				_U_(0x22)
#define	WNET_SRB_SSR_STAT_SOCK_IPRAW_Val			_U_(0x32)
#define	WNET_SRB_SSR_STAT_SOCK_MACRAW_Val			_U_(0x42)
//#define	WNET_SRB_SSR_STAT_SOCK_PPPOE_Val			_U_(0x5F)

#define	WNET_SRB_SSR_STAT_SOCK_SYNSENT_Val			_U_(0x15)
#define	WNET_SRB_SSR_STAT_SOCK_SYNRECV_Val			_U_(0x16)
#define	WNET_SRB_SSR_STAT_SOCK_FIN_WAIT_Val			_U_(0x18)
#define	WNET_SRB_SSR_STAT_SOCK_CLOSING_Val			_U_(0x1a)
#define	WNET_SRB_SSR_STAT_SOCK_TIME_WAIT_Val		_U_(0x1b)
#define	WNET_SRB_SSR_STAT_SOCK_LAST_ACK_Val			_U_(0x1d)


#define WNET_SRB_SSR_STAT_SOCK_CLOSED				(WNET_SRB_SSR_STAT_SOCK_CLOSED_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_INIT					(WNET_SRB_SSR_STAT_SOCK_INIT_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_LISTEN				(WNET_SRB_SSR_STAT_SOCK_LISTEN_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_ESTABLISHED			(WNET_SRB_SSR_STAT_SOCK_ESTABLISHED_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_CLOSE_WAIT			(WNET_SRB_SSR_STAT_SOCK_CLOSE_WAIT_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_IPRAW				(WNET_SRB_SSR_STAT_SOCK_IPRAW_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_UDP					(WNET_SRB_SSR_STAT_SOCK_UDP_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_MACRAW				(WNET_SRB_SSR_STAT_SOCK_MACRAW_Val << WNET_SRB_SSR_STAT_Pos)
//#define WNET_SRB_SSR_STAT_SOCK_PPPOE				(WNET_SRB_SSR_STAT_SOCK_PPPOE_Val << WNET_SRB_SSR_STAT_Pos)

#define WNET_SRB_SSR_STAT_SOCK_SYNSENT				(WNET_SRB_SSR_STAT_SOCK_SYNSENT_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_SYNRECV				(WNET_SRB_SSR_STAT_SOCK_SYNRECV_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_FIN_WAIT				(WNET_SRB_SSR_STAT_SOCK_FIN_WAIT_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_CLOSING				(WNET_SRB_SSR_STAT_SOCK_CLOSING_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_TIME_WAIT			(WNET_SRB_SSR_STAT_SOCK_TIME_WAIT_Val << WNET_SRB_SSR_STAT_Pos)
#define WNET_SRB_SSR_STAT_SOCK_LAST_ACK				(WNET_SRB_SSR_STAT_SOCK_LAST_ACK_Val << WNET_SRB_SSR_STAT_Pos)


// Socket Source Port Register
typedef union {
	uint16_t SPORT;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_SPORT_Type;		

#define WNET_SRB_SPORT_OFFSET		0x0004	// Offset "Socket Source Port Register"
#define WNET_SRB_SPORT_ADDRESS		0x0004


// Socket Destination Hardware Address Register
typedef union {
	WNET_MAC_Type DHAR;
} WNET_SRB_DHAR_Type;

#define WNET_SRB_DHAR_OFFSET		0x0006	// Offset "Socket Destination Hardware Address Register"
#define WNET_SRB_DHAR_ADDRESS		0x0006


// Socket Destination IP Address Register
typedef union {
	WNET_IP_Type DIPR;
} WNET_SRB_DIPR_Type;

#define WNET_SRB_DIPR_OFFSET		0x000c	// Offset "Socket Destination IP Address Register"
#define WNET_SRB_DIPR_ADDRESS		0x000c


// Socket Destination Port Register
typedef union {
	uint16_t DPORT;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_DPORT_Type;

#define WNET_SRB_DPORT_OFFSET		0x0010	// Offset "Socket Destination Port Register"
#define WNET_SRB_DPORT_ADDRESS		0x0010


// Socket Maximum Segment Size Register
typedef union {
	uint16_t MSSR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_MSSR_Type;

#define WNET_SRB_MSSR_OFFSET		0x0012	// Offset "Socket Maximum Segment Size Register"
#define WNET_SRB_MSSR_ADDRESS		0x0012


// Socket IP protocol Register
typedef union {
	struct {
		uint8_t IPPROTO :8;	// IP protocol
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_SIPPR_Type;

#define WNET_SRB_SIPPR_OFFSET						0x0014	// Offset "Socket Status Register"
#define WNET_SRB_SIPPR_ADDRESS						0x0014


#define WNET_SRB_SIPPR_IPPROTO_Pos					0			// Position "Socket Status Register"
#define WNET_SRB_SIPPR_IPPROTO_Msk					(_U_(0xff) << WNET_SRB_SSR_STAT_Pos)


#define	WNET_SRB_SIPPR_IPPROTO_IP_Val				_U_(0x00)	// Dummy for IP
#define	WNET_SRB_SIPPR_IPPROTO_ICMP_Val				_U_(0x01)	// Control message protocol
#define	WNET_SRB_SIPPR_IPPROTO_IGMP_Val				_U_(0x02)	// Internet group management protocol
#define	WNET_SRB_SIPPR_IPPROTO_GGP_Val				_U_(0x03)	// Gateway^2 (deprecated)
#define	WNET_SRB_SIPPR_IPPROTO_TCP_Val				_U_(0x06)	// TCP
#define	WNET_SRB_SIPPR_IPPROTO_PUP_Val				_U_(0x0c)	// PUP
#define	WNET_SRB_SIPPR_IPPROTO_UDP_Val				_U_(0x11)	// UDP
#define	WNET_SRB_SIPPR_IPPROTO_IDP_Val				_U_(0x16)	// XNS IDP
#define	WNET_SRB_SIPPR_IPPROTO_ND_Val				_U_(0x4d)	// UNOFFICIAL net disk protocol
#define	WNET_SRB_SIPPR_IPPROTO_RAW_Val				_U_(0xff)	// Raw IP packet


#define WNET_SRB_SIPPR_IPPROTO_IP					(WNET_SRB_SIPPR_IPPROTO_IP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_ICMP					(WNET_SRB_SIPPR_IPPROTO_ICMP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_IGMP					(WNET_SRB_SIPPR_IPPROTO_IGMP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_GGP					(WNET_SRB_SIPPR_IPPROTO_GGP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_TCP					(WNET_SRB_SIPPR_IPPROTO_TCP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_PUP					(WNET_SRB_SIPPR_IPPROTO_PUP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_UDP					(WNET_SRB_SIPPR_IPPROTO_UDP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_IDP					(WNET_SRB_SIPPR_IPPROTO_IDP_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_ND					(WNET_SRB_SIPPR_IPPROTO_ND_Val << WNET_SRB_SIPPR_IPPROTO_Pos)
#define WNET_SRB_SIPPR_IPPROTO_RAW					(WNET_SRB_SIPPR_IPPROTO_RAW_Val << WNET_SRB_SIPPR_IPPROTO_Pos)


// Socket IP Type of Service Register
typedef union {
	uint8_t TOS;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_TOS_Type;

#define WNET_SRB_TOS_OFFSET			0x0015	// Offset "Socket IP Type of Service Register"
#define WNET_SRB_TOS_ADDRESS		0x0015


// Socket TTL Register
typedef union {
	uint8_t TTL;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_TTL_Type;

#define WNET_SRB_TTL_OFFSET			0x0016	// Offset "Socket TTL Register"
#define WNET_SRB_TTL_ADDRESS		0x0016


// Socket RX Buffer Size Register
typedef union {
	uint8_t RXBUF_SIZE;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_RXBUF_SIZE_Type;

#define WNET_SRB_RXBUF_SIZE_OFFSET			0x001e	// Offset "Socket RX Buffer Size Register"
#define WNET_SRB_RXBUF_SIZE_ADDRESS			0x001e


#define	WNET_SRB_RXBUF_SIZE_0KB				_U_(0x00)	
#define	WNET_SRB_RXBUF_SIZE_1KB				_U_(0x01)
#define	WNET_SRB_RXBUF_SIZE_2KB				_U_(0x02)
#define	WNET_SRB_RXBUF_SIZE_4KB				_U_(0x04)
#define	WNET_SRB_RXBUF_SIZE_8KB				_U_(0x08)
#define	WNET_SRB_RXBUF_SIZE_16KB			_U_(0x10)


// Socket TX Buffer Size Register
typedef union {
	uint8_t TXBUF_SIZE;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_TXBUF_SIZE_Type;

#define WNET_SRB_TXBUF_SIZE_OFFSET			0x001f	// Offset "Socket TX Buffer Size Register"
#define WNET_SRB_TXBUF_SIZE_ADDRESS			0x001f


#define	WNET_SRB_TXBUF_SIZE_0KB				_U_(0x00)
#define	WNET_SRB_TXBUF_SIZE_1KB				_U_(0x01)
#define	WNET_SRB_TXBUF_SIZE_2KB				_U_(0x02)
#define	WNET_SRB_TXBUF_SIZE_4KB				_U_(0x04)
#define	WNET_SRB_TXBUF_SIZE_8KB				_U_(0x08)
#define	WNET_SRB_TXBUF_SIZE_16KB			_U_(0x10)


// Socket TX Free Size Register
typedef union {
	uint16_t TX_FSR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_TX_FSR_Type;

#define WNET_SRB_TX_FSR_OFFSET		0x0020	// Offset "Socket TX Free Size Register"
#define WNET_SRB_TX_FSR_ADDRESS		0x0020


// Socket TX Read Pointer Register
typedef union {
	uint16_t TX_RD;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_TX_RD_Type;

#define WNET_SRB_TX_RD_OFFSET		0x0022	// Offset "Socket TX Read Pointer Register"
#define WNET_SRB_TX_RD_ADDRESS		0x0022


// Socket TX Write Pointer Register
typedef union {
	uint16_t TX_WR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_TX_WR_Type;

#define WNET_SRB_TX_WR_OFFSET		0x0024	// Offset "Socket TX Write Pointer Register"
#define WNET_SRB_TX_WR_ADDRESS		0x0024


// Socket Received Size Register
typedef union {
	uint16_t RX_RSR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_RX_RSR_Type;

#define WNET_SRB_RX_RSR_OFFSET		0x0026	// Offset "Socket Received Size Register"
#define WNET_SRB_RX_RSR_ADDRESS		0x0026


// Socket RX Read Data Pointer Register
typedef union {
	uint16_t RX_RD;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_RX_RD_Type;

#define WNET_SRB_RX_RD_OFFSET		0x0028	// Offset "Socket RX Read Data Pointer Register"
#define WNET_SRB_RX_RD_ADDRESS		0x0028


// Socket RX Write Pointer Register
typedef union {
	uint16_t RX_WR;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_RX_WR_Type;

#define WNET_SRB_RX_WR_OFFSET		0x002a	// Offset "Socket RX Write Pointer Register"
#define WNET_SRB_RX_WR_ADDRESS		0x002a


// Socket Interrupt Mask Register
typedef union {
	struct {
		uint8_t CON     :1;		// Connection with peer
		uint8_t DISCON  :1;		// This is issued when FIN or FIN/ACK packet is received from a peer
		uint8_t RECV    :1;		// This is issued whenever data is received from a peer
		uint8_t TIMEOUT :1;		// This is issued when ARPTO or TCPTO occurs.
		uint8_t SEND_OK :1;		// This is issued when SEND command is completed.
		uint8_t	        :3;		// Res.
	} b;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_IMR_Type;

#define WNET_SRB_IMR_OFFSET				0x002c	// Offset "Socket Interrupt Mask Register"
#define WNET_SRB_IMR_ADDRESS			0x002c


#define WNET_SRB_IMR_CON_Pos			0		// Position "Connection"
#define WNET_SRB_IMR_CON				(_U_(0x1) << WNET_SRB_IMR_CON_Pos)

#define WNET_SRB_IMR_DICON_Pos			1		// Position "Disconnection"
#define WNET_SRB_IMR_DICON				(_U_(0x1) << WNET_SRB_IMR_DICON_Pos)

#define WNET_SRB_IMR_RECV_Pos			2		// Position "Receive"
#define WNET_SRB_IMR_RECV				(_U_(0x1) << WNET_SRB_IMR_RECV_Pos)

#define WNET_SRB_IMR_TIMEOUT_Pos		3		// Position "Timeout"
#define WNET_SRB_IMR_TIMEOUT			(_U_(0x1) << WNET_SRB_IMR_TIMEOUT_Pos)

#define WNET_SRB_IMR_SEND_OK_Pos		4		// Position "Send OK"
#define WNET_SRB_IMR_SEND_OK			(_U_(0x1) << WNET_SRB_IMR_SEND_OK_Pos)


// Socket Fragment Register
typedef union {
	uint16_t FRAG;
	uint8_t  BYTE[2];
	uint16_t hw;
} WNET_SRB_FRAG_Type;

#define WNET_SRB_FRAG_OFFSET		0x002d	// Offset "Socket Fragment Register"
#define WNET_SRB_FRAG_ADDRESS		0x002d


// Socket Keep Alive Time Register
typedef union {
	uint8_t KPALVTR;
	uint8_t BYTE[1];
	uint8_t by;
} WNET_SRB_KPALVTR_Type;

#define WNET_SRB_KPALVTR_OFFSET		0x002f	// Offset "Socket Fragment Register"
#define WNET_SRB_KPALVTR_ADDRESS	0x002f



#endif /* W5500SRB_H_ */