/*
 * main.c
 *
 * Created: 29.08.2018 18:31:47
 * Author : Thomas F
 */ 


#include <stdbool.h>
#include <string.h>

#include "sam.h"

#include "clocks.h"
#include "IO.h"
#include "misc.h"
#include "Time/Time.h"
#include "StatusLed/StatusLed.h"
#include "DMA/dma.h"
#include "Internet/Internet.h"
#include "SocketMain/SocketMain.h"
#include "Router/Router.h"
#include "StatusLed/StatusLed.h"



#define WDT_RESET	do { if (WDT->SYNCBUSY.bit.CLEAR == false) { WDT->CLEAR.reg = WDT_CLEAR_CLEAR_KEY; } } while(0);



int main(void)
{
	// Initialize the SAM system
    SystemInit();
	SystemInitClock();


	// The detection of a debugger probe could fail if BOD33 is enabled
	#ifdef DEBUG
		SUPC->BOD33.bit.ENABLE = false;
	#endif


	// Enable Cortex M Cache Controller
	CMCC->CTRL.reg = CMCC_CTRL_CEN;


	// Enable watchdog if not already enabled
	if (!WDT->CTRLA.bit.ENABLE)
	{
		WDT->CONFIG.reg = WDT_CONFIG_PER_CYC512; // 512 / 1024Hz = 500ms
	
		WDT->CTRLA.reg = WDT_CTRLA_ENABLE;
		while (WDT->SYNCBUSY.bit.ENABLE) { ; }
	}


	// Initialize Timer
	TimeInit();
	WDT_RESET;
		

	// Check reset reason
	_Bool ErrorAbnormalReset = (RSTC->RCAUSE.bit.POR == false && RSTC->RCAUSE.bit.EXT == false);
	
	if (ErrorAbnormalReset) { Delay_ms(5000); }


	// Initialize all functions
	StatusLedInit();
	WDT_RESET;
	
	DmaInit();
	WDT_RESET;
 	
	WNetInit();
	WDT_RESET;
	
	RouterInit();
	WDT_RESET;

	SocketMainInit();
	WDT_RESET;
	
	
	// Main loop
    while (1) 
    {
		if(SocketMain() < 0) { StatusLedSignalError(); }
				
		if (ErrorAbnormalReset) { StatusLedSignalError(); }
				
		StatusLedMain();

		WDT_RESET;
    }

}

