/*
 * misc.h
 *
 * Created: 04.12.2017 16:50:49
 *  Author: Thomas F
 */ 


#ifndef MISC_H_
#define MISC_H_


// ########## Global const

#define INT_PRIO_BG_L	(5)	// Background low
#define INT_PRIO_BG_M	(4)	// Background medium
#define INT_PRIO_BG_H	(3)	// Background high
#define INT_PRIO_RT_L	(2)	// Real time low
#define INT_PRIO_RT_M	(1)	// Real time medium
#define INT_PRIO_RT_H	(0)	// Real time high


// ########## Global #define


#define OPTIMIZATION_BARRIER	__DMB();	//__ASM volatile ("dmb" : : : "memory");


// ########## Function

#define RAM_FUNC	__attribute__ ((long_call, section (".ramfunc")))
#define FLASH_FUNC
#define INLINE_FUNC	static inline __attribute__((always_inline))


// ########## Sections

#define RAM		__attribute__ ((section(".sram")))
#define FLASH	__attribute__ ((section(".flash")))
#define BKUPRAM	__attribute__ ((section(".bkupram")))
#define QSPIRAM	__attribute__ ((section(".qspi")))


// ########## IRQ

#define ISR(func) void func(void)	// Cortex Interrupt handlers can be programmed as normal C functions 

INLINE_FUNC void DisableIRQ(void)
{
	__disable_irq();
	__DMB();
}

INLINE_FUNC void EnableIRQ(void)
{
	__DMB();
	__enable_irq();
}

INLINE_FUNC void EnterCriticalSectionAtomic(uint32_t* atomic)
{
	*atomic = __get_PRIMASK();
	__disable_irq();
	__DMB();
}

INLINE_FUNC void LeaveCriticalSectionAtomic(uint32_t* atomic)
{
	__DMB();
	__set_PRIMASK(*atomic);
}



INLINE_FUNC uint32_t EnterAtomicBlock(void)
{
	uint32_t atomic = __get_PRIMASK();
	__disable_irq();
	__DMB();
	return atomic;
}

INLINE_FUNC uint32_t LeaveAtomicBlock(uint32_t* atomic)
{
	__DMB();
	__set_PRIMASK(*atomic);
	
	return 0;
}

#define ATOMIC_BLOCK(void)	for(	uint32_t atomicBlockOnce = 1, atomicBlockPrimask = EnterAtomicBlock(); \
									atomicBlockOnce ; \
									atomicBlockOnce = LeaveAtomicBlock(&atomicBlockPrimask) \
								)


// ########## PORT

#define PORTA           ((PortGroup*)&PORT->Group[0])
#define PORTB           ((PortGroup*)&PORT->Group[1])
//#define PORTC           ((PortGroup*)&PORT->Group[2])

#define PMUX_A	0
#define PMUX_B	1
#define PMUX_C	2
#define PMUX_D	3
#define PMUX_E	4
#define PMUX_F	5
#define PMUX_G	6
#define PMUX_H	7
#define PMUX_I	8
#define PMUX_J	9
#define PMUX_K	10
#define PMUX_L	11
#define PMUX_M	12
#define PMUX_N	13


#endif /* MISC_H_ */